/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/* Program Name: PT-KB.P
 * Author      : Gerry Seidl
 * Written on  : January 15, 1993
 * Description : Maintain Key Bindings
 */

{proterm/pt-share.i} /* Global Defs */

DEFINE VAR list-status as logical.

/* Definitions of the field level widgets                                   */
DEFINE VARIABLE SEL-KL AS CHARACTER 
    VIEW-AS SELECTION-LIST SINGLE 
     SIZE-CHAR 23 BY 6                    NO-UNDO.

DEFINE VARIABLE SEL-KF AS CHARACTER 
    VIEW-AS SELECTION-LIST SINGLE 
     SIZE-CHAR 23 BY 6                    NO-UNDO.

DEFINE NEW SHARED VARIABLE tmp-kl AS CHARACTER FORMAT "X(20)" 
    LABEL "&Key Label"                    NO-UNDO.

DEFINE NEW SHARED VARIABLE tmp-escseq AS CHARACTER FORMAT "X(50)" 
    LABEL "&Esc Seq"  NO-UNDO.

DEFINE BUTTON Save-But
     LABEL "&Save"
     SIZE-CHAR 10 BY 1 .

DEFINE BUTTON Add-But
     LABEL "&Add"
     SIZE-CHAR 10 BY 1 .

DEFINE BUTTON MMenu-but
     LABEL "&Main Menu"
     SIZE-CHAR 12 BY 1 .

DEFINE VARIABLE EscSeq AS CHARACTER FORMAT "X(22)" 
     LABEL "&Esc Seq"  NO-UNDO.

DEFINE BUTTON Del-But
     LABEL "&Delete"
     SIZE-CHAR 10 BY 1 .


/* Definitions of the frame widgets                                */
DEFINE FRAME FRAME-A
     "Key Functions" VIEW-AS TEXT AT ROW 2 COL 9
     "Defined Key Labels" VIEW-AS TEXT AT ROW 2 COL 32
     SEL-KF AT ROW 3 COL 5 NO-LABEL
     SEL-KL AT ROW 3 COL 30 NO-LABEL
     Add-But AT ROW 3 COL 55
     Del-But AT ROW 5 COL 55
     Save-But AT ROW 7 COL 55
     EscSeq AT ROW 10 COL 12.72
          LABEL "Esc Seq"  FORMAT "X(10)" 
     MMenu-but AT ROW 10 COL 55
     WITH  1 DOWN OVERLAY SIDE-LABELS
       AT COL 4 ROW 4
       SIZE-CHAR 69 BY 13 
       TITLE "Keybindings for terminal " + termtype.

ON CHOOSE OF MMenu-But DO:
   HIDE FRAME FRAME-A.
   RETURN.
END.

ON VALUE-CHANGED OF SEL-KF DO:
   sel-kl:list-items in frame frame-a = "".
   sel-kl:delimiter in frame frame-a = "*".
   for each kbind where kf = sel-kf:screen-value in frame frame-a:
       list-status = sel-kl:add-last(kbind.kl).
   end.
   if sel-kl:list-items = ? then
   do:
        display sel-kl with frame frame-a.
        disable sel-kl with frame frame-a.
   end.
   else enable sel-kl with frame frame-a.
   find first kbind where kf = sel-kf:screen-value in frame frame-a no-error.
   if available (kbind) then do:
       sel-kl:screen-value = kbind.kl.
       ESCSEQ = KBIND.K-ESCSEQ.
       DISPLAY ESCSEQ WITH FRAME FRAME-A.
   END.
   ELSE DO:
       ESCSEQ = "".
       DISPLAY ESCSEQ WITH FRAME FRAME-A.
   END.
end.

ON VALUE-CHANGED OF SEL-KL DO:
   FIND first KBIND where
       kbind.kf = sel-kf:screen-value in frame frame-a and
       kbind.kl = sel-kl:screen-value in frame frame-a no-error.
   if available (kbind) then do:
       assign escseq = k-escseq.
       display escseq with frame frame-a.
   end.
end.

ON CHOOSE OF Del-But DO:
   IF SEL-KL:SCREEN-VALUE IN FRAME FRAME-A = "" THEN DO:
       MESSAGE "You must select a Key Label to delete." view-as alert-box
           error buttons ok.
       RETURN NO-APPLY.
   END.
   ELSE DO:
       DEF VAR CHOICE AS LOGICAL.
       MESSAGE "Are you sure you want to delete"
             SEL-KL:SCREEN-VALUE IN FRAME FRAME-A "?" view-as
             alert-box warning buttons yes-no update choice.
       if choice then do:
         ASSIGN TMP-KL = SEL-KL:SCREEN-VALUE IN FRAME FRAME-A. /* SAVE OFF */
         FIND first KBIND where
            kbind.kf = sel-kf:screen-value in frame frame-a and
            kbind.kl = sel-kl:screen-value in frame frame-a no-error.
         if available (kbind) then DELETE KBIND.
         list-status = SEL-KL:DELETE(TMP-KL). /* delete from sel list */
         IF SEL-KL:LIST-ITEMS = ? THEN DISABLE SEL-KL WITH FRAME FRAME-A.
         ESCSEQ = "".  /* clear out field */
         DISPLAY ESCSEQ WITH FRAME FRAME-A. 
       END.
       ELSE RETURN NO-APPLY.
   END.
END.                       

ON CHOOSE OF Add-But DO:
   IF SEL-KF:SCREEN-VALUE = "" THEN DO:
       MESSAGE "You must select a Key Function first!" VIEW-AS ALERT-BOX
            ERROR BUTTONS OK.
       APPLY "ENTRY" TO SEL-KF IN FRAME FRAME-A.
       RETURN NO-APPLY.
   END.
   tmp-kl = "".
   tmp-escseq = "".
   repeat:
      DEF VAR stat AS INT INITIAL 1. /* 0 = ok, 1 = Cancel */
      run pt-kld.p (OUTPUT stat).
      IF stat = 1 then DO:
          /* User hit Cancel in Dialog Box */
          MESSAGE "Add operation aborted" VIEW-AS ALERT-BOX INFORMATION
               BUTTONS OK.
          RETURN NO-APPLY.
      END.
      IF TMP-KL <> "" AND TMP-ESCSEQ <> "" THEN DO:
         if index(tmp-kl,"ALT") <> 0 or index(tmp-kl,"MOUSE") <> 0 then do:
            message "You may not use a Key Label" skip
                    "that contains ALT or MOUSE." view-as alert-box error
                    buttons ok.
            tmp-kl = "".
            tmp-escseq = "".
            next.
         end.
         find first kbind where kl = tmp-kl no-error.
         if available(kbind) then do:
            message "The Key Function" kbind.kf "already has" skip
                 tmp-kl "defined for it." view-as alert-box error
                 buttons ok.
            tmp-kl = "".
            tmp-escseq = "".
            next.
         end.
         ENABLE SEL-KL WITH FRAME FRAME-A.
         /* Fix Tabbing of Enabled widgets */
         ASSIGN SEL-KF:TAB-POSITION    = 1
                SEL-KL:TAB-POSITION    = 2
                ADD-BUT:TAB-POSITION   = 3
                DEL-BUT:TAB-POSITION   = 4
                SAVE-BUT:TAB-POSITION  = 5
                ESCSEQ:TAB-POSITION    = 6
                MMenu-But:TAB-POSITION = 7.
         list-status = sel-kl:add-last(tmp-kl) in frame frame-a.
         sel-kl:screen-value = tmp-kl.
         ESCSEQ = tmp-escseq.
         DISPLAY ESCSEQ WITH FRAME FRAME-A.
         create kbind.
         assign kbind.kf = sel-kf:screen-value in frame frame-a.
         assign kbind.kl = sel-kl:screen-value in frame frame-a.
         assign kbind.k-escseq = tmp-escseq.
         release kbind.
         LEAVE.
      END.
      ELSE LEAVE.
   END. /* REPEAT */
end.

ON CHOOSE OF Save-But DO:
   IF SEL-KF:SCREEN-VALUE IN FRAME FRAME-A = "" then DO:
       message "You must select a Key Function to save" view-as alert-box
           error buttons ok.
       RETURN NO-APPLY.
   END.
   ELSE IF SEL-KF:SCREEN-VALUE IN FRAME FRAME-A = "" then DO:
       message "You must select a Key Label to save" view-as alert-box
           error buttons ok.
       RETURN NO-APPLY.
   END.
   ELSE IF ESCSEQ:SCREEN-VALUE IN FRAME FRAME-A = "" then DO:
       message "You must enter an Escape Sequence for this" skip
               "Key Function / Key Label combination." view-as alert-box
               error buttons ok.
       return no-apply.
   END.
   ELSE DO:
       find first KBIND where
           kbind.kf = sel-kf:screen-value in frame frame-a and
           kbind.kl = sel-kl:screen-value in frame frame-a no-error.
       if available (KBIND) then
           assign K-ESCSEQ = ESCSEQ:SCREEN-VALUE IN FRAME FRAME-A.
       ELSE DO:
           create KBIND.
           ASSIGN kbind.kf = sel-kf:screen-value in frame frame-a
                  kbind.kl = sel-kl:screen-value in frame frame-a
                  k-escseq = ESCSEQ:SCREEN-VALUE IN FRAME FRAME-A.
       END.
       RELEASE KBIND.
   END.
END.                                              
DISPLAY 
     SEL-KF
     SEL-KL
     Add-But
     Del-But
     Save-But
     EscSeq
     MMenu-but
    WITH FRAME FRAME-A.
ENABLE ALL EXCEPT SEL-KL WITH FRAME FRAME-A.

/* load up keyfunctions */
for each key_functions BY key_function:
    list-status = SEL-KF:add-last(key_function.key_function) 
          in frame frame-a.
    if not list-status then undo, return.
end.

/* Position KF selection list to first entry */
FIND FIRST KEY_FUNCTIONS NO-ERROR.
IF AVAILABLE(KEY_FUNCTIONS) THEN
     SEL-KF:SCREEN-VALUE = KEY_FUNCTIONS.KEY_FUNCTION.
ELSE DO:
    MESSAGE "The Key Functions file appears to be empty!" CHR(10)
        "This function cannot proceed" VIEW-AS ALERT-BOX ERROR BUTTONS OK.
    RETURN.
END.

/* ASSIGN RUN TIME ATTRIBUTES                                  */

/* Define temporary widget handle for run-time attributes    */
DEFINE VARIABLE tmp_handle AS WIDGET-HANDLE NO-UNDO.

ASSIGN tmp_handle = EscSeq:HANDLE IN FRAME FRAME-A
       tmp_handle:WIDTH-CHARS                   = 25.


WAIT-FOR CHOOSE OF MMenu-but FOCUS SEL-KF.

/* pt-kb.p - end of file */
