/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/* Program Name: PT-MENU.P
 * Author      : Gerry Seidl
 * Written on  : January 15, 1993
 * Description : Main Menu that allows manipulation of protermcap entry
 */

{proterm/pt-share.i} /* GLOBAL STUFF */ 

/* Definitions of the field level widgets                                    */
DEFINE BUTTON BUTTON-4
     LABEL "&Preview"
     SIZE-CHAR 13.14 BY 1 .

DEFINE BUTTON BUTTON-6
     LABEL "&Exit"
     SIZE-CHAR 10 BY 1 .

DEFINE RECTANGLE RECT-1
     SIZE-CHAR 47.86 BY 11 
     EDGE-PIXELS 1 NO-FILL .

DEFINE BUTTON BUTTON-1
     LABEL "&Colors"
     SIZE-CHAR 13.14 BY 1 .

DEFINE BUTTON BUTTON-2
     LABEL "&Keybindings"
     SIZE-CHAR 13.14 BY 1 .

DEFINE BUTTON BUTTON-3
     LABEL "&Term. Caps."
     SIZE-CHAR 13.14 BY 1 .

/* Definitions of the frame widgets                   */
DEFINE FRAME FRAME-B
     RECT-1 AT ROW 1 COL 3
     BUTTON-2 AT ROW 3 COL 5
     "Assign KeyFunctions to Keys" VIEW-AS TEXT
           AT ROW 3 COL 20
     BUTTON-1 AT ROW 5 COL 5
     "Define Colors" VIEW-AS TEXT
           AT ROW 5 COL 20
     BUTTON-6 AT ROW 5 COL 54
     BUTTON-3 AT ROW 7 COL 5
     "Assign Teminal Capabilities" VIEW-AS TEXT
           AT ROW 7 COL 20
     BUTTON-4 AT ROW 9 COL 5
     "View current protermcap entry" VIEW-AS TEXT
           AT ROW 9 COL 20
    WITH  1 DOWN OVERLAY     SIDE-LABELS
        AT COL 4 ROW 3
        SIZE-CHAR 72 BY 14 
        TITLE "Main Menu".

ON CHOOSE OF BUTTON-6 DO:
    HIDE FRAME FRAME-B.
    RETURN.
END.

ON CHOOSE OF BUTTON-4 DO:
    HIDE FRAME FRAME-B.
    RUN "pt-prev.p".
END.

ON CHOOSE OF BUTTON-1 DO:
    HIDE FRAME FRAME-B.
    RUN "pt-color.p".
END.

ON CHOOSE OF BUTTON-2 DO:
    DEF VAR CHOICE AS LOGICAL INITIAL NO.
    HIDE FRAME FRAME-B.
    RUN "pt-escd.p". /* Get ESC char for new terminal */
    if escval = "" then return no-apply.
    if not defaultkb then do:
          message "Do you want to load the default Key Bindings?" skip
                  "(They will work on most terminals)" view-as alert-box
                  question buttons yes-no update choice.
          if choice then do:
             assign termfile = termtype + ".imp".
             os-delete value(termfile).
             assign termfile = "v7kf.out".
             DEF VAR crt_stat AS INT INITIAL 1.
             INPUT THROUGH crtmain protermcap '""' v7kf no-echo.
                SET crt_stat WITH FRAME res.
             INPUT CLOSE.
             if crt_stat = 0 then do:
                 run readentry. /* read it in */
             end.
             else do:
                 message "Default Key Bindings cannot be loaded!"  chr(10)
                         "An error occurred in crtmain. Make sure" chr(10)
                         "that the 'v7kf' entry exists in file "   chr(10)
                         "specified by $PROTERMCAP."
                         view-as alert-box warning buttons ok.
             end.
           end. /* if choice */
    end.
    assign defaultkb = true.
    RUN "pt-kb.p".
END.

ON CHOOSE OF BUTTON-3 DO:
    HIDE FRAME FRAME-B.
    RUN "pt-tcap.p".
END.

DISPLAY 
     RECT-1
     BUTTON-2
     BUTTON-1
     BUTTON-6
     BUTTON-3
     BUTTON-4
    WITH FRAME FRAME-B.
DISPLAY termtype AT 37 NO-LABEL WITH FRAME o OVERLAY ROW 21 NO-BOX.
ENABLE ALL WITH FRAME FRAME-B.

WAIT-FOR CHOOSE OF BUTTON-6 FOCUS BUTTON-2.

PROCEDURE readentry.
    def var termorig like termfile.
    def var titleorig like titleline.
    def var tctimes as int.
    assign termorig = termfile. /* save off old termfile */
    assign titleorig = titleline.
    tctimes = 0.
    repeat:
      run pt-read.p.
      if escval <> "" and escval <> "\E" then do:   /* fix ESC */
         for each kbind:
             assign k-escseq = replace(k-escseq, "\E", escval).
         end.
      end.
      tctimes = tctimes + 1.
      find first tcap where code = "tc" no-error.
      if available (tcap) then do:   /* need to read another file */
         if t-escseq = "v7kf" then assign defaultkb = true.
         message "A 'tc' entry was read that points to an entry for"
             t-escseq skip "This file will now be read in." view-as 
             alert-box information buttons ok.
           TERMFILE = t-escseq + ".out". /* filename to read in */
           /* create the entry on disk to read in */
           def var rescode as int INITIAL 1. /* status of crtmain 0/1 */
           CASE NEWFROM:
           WHEN 1 THEN DO:
             INPUT THROUGH crtmain termcap '""' value(t-escseq) NO-ECHO.
                 SET RESCODE WITH FRAME res.
             INPUT CLOSE.
           END.
           WHEN 2 THEN DO:
             INPUT THROUGH crtmain terminfo '""' value(t-escseq) NO-ECHO.
                 SET RESCODE WITH FRAME res.
             INPUT CLOSE.
           END.
           WHEN 3 THEN DO:
             INPUT THROUGH crtmain protermcap '""' value(t-escseq) NO-ECHO.
                 SET RESCODE WITH FRAME res.
             INPUT CLOSE.
           END.
           END CASE.
           if RESCODE <> 0 then do:
                  view frame frame-a.
                  message "An error occurred in crtmain. Creation aborted"
                      view-as alert-box error
                      buttons ok.
                  return error.
           end.
      end. /* if available */
      else do:
          if termorig <> "" then termfile = termorig.
          if titleorig <> "" then titleline = titleorig.
          leave.
      end.
    end. /* repeat */ 
END.

/* pt-menu.p - end of file */
