/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/* Program Name: PT-READ.P
 * Author      : Gerry Seidl
 * Written on  : January 15, 1993
 * Description : Read in entry
 */

{proterm/pt-share.i} /* GLOBAL DEFS */

DEFINE VARIABLE ERRFLAG AS LOGICAL INITIAL FALSE. /* records if err occur*/
DEFINE VARIABLE tline AS CHARACTER FORMAT "X(128)". /* holds input line */
DEFINE VARIABLE c AS INTEGER. /* misc var */
DEFINE VAR slash AS INT.
DEFINE VAR eqsign AS INT.
DEFINE VAR x AS INT.
DEFINE VAR g AS CHAR format "X(1)".
DEFINE VAR openp AS INT.
DEFINE VAR closep AS INT.
DEFINE VAR nums AS CHARACTER INITIAL "1,2,3,4,5,6,7,8,9".
DEFINE VAR V6color AS LOGICAL INITIAL false.
DEFINE VAR tmp-cno AS INTEGER INITIAL 5.

/* Definitions of the frame widgets */
DEFINE FRAME meter-1
     "Reading file:" AT ROW 2 COL 2
     termfile at row 4 col 2 no-label
    WITH VIEW-AS DIALOG-BOX SIDE-LABELS SIZE-CHAR 50 BY 7  overlay
        TITLE "Status of Input".

enable all with frame meter-1.
display termfile  format "X(45)" view-as text with frame meter-1.
DEFINE STREAM ERR.
pause 0 before-hide.
find first tcap where code = "tc" no-error.
if available (tcap) then do:
    delete tcap.  
end.
output stream err to err.out APPEND.
input from value(termfile).
MAIN:
repeat:
    import tline.
    tline = trim(tline). /* hack off spaces and TABs */
    if tline = "" then next. /* skip blank lines */
    repeat:
     case keycode(substring(tline,1,1)):
     when 9 then
         tline = substring(tline,2). /* drop TAB */                         
        otherwise
         leave.
        end case.
    end.    
    if substring(tline,1,1) = "." then next. /* commented out */
    if index(tline, "|") <> 0 and substring(tline,1,5) <> "COLOR" then do:
        /* title line of protermcap */
        if newstat <> 2 then assign titleline = tline.
        next.
    end.
    else if index(tline, "#") <> 0 then do:
        /* probably CO or LI */
        find first tcap where code = substring(tline,1,2) no-error.
        if not available(tcap) then do:
           create tcap.
           assign code = substring(tline,1,2) 
                  t-escseq = substring(tline,4).
           release tcap.
        end.
        next.
    end.
    else if index(tline, "=") = 3 or 
         length(tline) = 2 then do:
            /* terminal capability */
            if substring(tline,1,2) <> "tc" then do:
               find first tcap where code = substring(tline,1,2) no-error.
               if not available (tcap) then do:
                  create tcap.
                  assign code = substring(tline, 1, 2).
                  if length(tline) > 2 then t-escseq = substring(tline, 4).
                  else t-escseq = "<set>".
                  release tcap.
               end.
            end. 
            else do: /* 'tc', create or overwrite */
               find first tcap where code = "tc" no-error.
               if available (tcap) then
                   assign t-escseq = substring(tline,4).   
               else do:
                   create tcap.
                   assign code = "tc"
                          t-escseq = substring(tline,4).
               end.    
            end.
            next.
    end.
    else if substring(tline,1,5) = "COLOR" then do:
        /* check for color number, if none, then V6 protermcap entry */
        if lookup(substring(tline,7,1),nums) = 0 then V6color = true.
        else V6color = false.
        slash = index(tline, "/").
        eqsign = index(tline, "=").
        if eqsign = 0 then do:
           /* there's something wrong with this Def */
           put stream err tline skip.
           assign errflag = true.
           next.
        end.   
     /*   create colors. */
        repeat x = 7 to eqsign:
           g = substring(tline,x,1).
           if g = " " or g = "=" then do:
               if V6color then do:
                  find last colors no-error.
                  if available (colors) then do:
                     tmp-cno = colors.cno + 1. /* to use for next color */
                  end.
                  else tmp-cno = 5. /* starting color is 5 */
               end.
               else tmp-cno = int(substring(tline,7,x - 7)). /* V7 color */
               find first colors where cno = tmp-cno no-error.
               if not available (colors) then do:
                   create colors.
                   cno = tmp-cno.
                   assign c-escseq = substring(tline,eqsign + 1).
                   leave.
               end.
               else next MAIN. /* skip this entry, becuase it's a dupe */
           end.
        end.
        IF V6color then x = 6. /* reset, becuase no color # */
        find first colors where cno = tmp-cno no-error.
        if slash > 0 then do: /* FG and BG defined */
            fg = substring(tline,x + 1,slash - x - 1).
            bg = substring(tline,slash + 1,eqsign - slash - 1).
        end.
        else do:
            /* only FG defined */
            if g = " " or V6color then 
                   fg = substring(tline,x + 1,eqsign - x - 1).
            else do: /* not literal colors defined */
               fg = "".
               bg = "".
            end.
        end.
        release tcap.
        next.                           
    end.
    else if index(tline,"(",1) <> 0 then DO: /* look for '(' */
        /* PROCESS KEYBINDINGS */
        openp = index(tline,"(",1).
        closep = index(tline,")",openp).
        eqsign = index(tline,"=",closep).
        if openp = 0 or
           closep = 0 or
           closep - openp < 3 or
           eqsign = 0 or
           eqsign - closep <> 1 then do:
              /* bad Def */
              put stream err tline skip.
              assign errflag = true.
              next.
        end.
        find first kbind where kl = 
             substring(tline,openp + 1, closep - openp - 1) no-error.
        if not available(kbind) then do:
            create kbind.
            assign kl = substring(tline,openp + 1, closep - openp - 1).
            if openp > 1 then  /* smells like a kf */
               assign kf = substring(tline,1,openp - 1).
            else kf = "".
            assign k-escseq = substring(tline,eqsign + 1).
        end.
        release kbind.
        next.
    end.
    /* line we can't handle */
    put stream err tline skip.                       
    assign errflag = true.
end.
input close.
output close.
hide frame meter-1.
if errflag then
    message "There were errors during input of " + TERMFILE + "." +
    " Please look at err.out" view-as alert-box error buttons ok.
def var chkout as char.
assign chkout = termtype + ".imp".
output to value(chkout).
for each tcap:
    display code t-escseq with width 200.
end.
for each kbind:
    display kbind.
end.
for each colors:
    display colors.
end.
output close.
message "View the file " + CHKOUT  + " for a log of the import."
    view-as alert-box information buttons ok.
return.

/* pt-read.p - end of file */
