/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/* Program Name: PT-WRITE.P
 * Author      : Gerry Seidl
 * Written on  : January 15, 1993
 * Description : Write out the protermcap entry as it's currently in
 *               memory
 */
{proterm/pt-share.i}

DEFINE VARIABLE c-tcchk AS CHARACTER INITIAL 
    "cm,cl,us,ue,so,se,li,co,ws,G1,G2,G3,G4,GV,GH" CASE-SENSITIVE.
DEFINE VARIABLE m-tcchk AS CHARACTER FORMAT "X(60)".
DEFINE VARIABLE a-tcchk AS INTEGER   EXTENT 15.
DEFINE VARIABLE ctr     AS INTEGER.

DEFINE STREAM TOUT.

OUTPUT STREAM TOUT TO VALUE(TERMOUT).

PUT STREAM TOUT UNFORMATTED 
  "# " TERMTYPE " entry." skip.
PUT STREAM TOUT UNFORMATTED 
  "# Created by the PROGRESS Protermcap Builder on " TODAY skip.
PUT STREAM TOUT UNFORMATTED 
  trim(titleline) skip.
  
FOR EACH TCAP BY CODE:
  IF LOOKUP(code, c-tcchk) <> 0 THEN
    a-tcchk[lookup(code, c-tcchk)] = 1. /* flag found */
  IF CODE = "tc" THEN NEXT. /* don't write out any tc's */
  IF CODE = "ti" OR CODE = "te" THEN NEXT. /* don't write per Chip */
  IF T-ESCSEQ = "<set>" THEN DO:
    PUT STREAM TOUT UNFORMATTED ":" CODE ":~\" skip.
    NEXT.
  END. 
  IF T-ESCSEQ <> "" THEN DO:
    IF CODE = "co" OR CODE = "li" THEN
      PUT STREAM TOUT UNFORMATTED ":" CODE "#" T-ESCSEQ ":~\" skip.
    ELSE 
      PUT STREAM TOUT UNFORMATTED ":" CODE "=" T-ESCSEQ ":~\" skip.
  END. 
END.

/* we can have 'ws' or ('co' and 'li'), so let's see what we have */
IF a-tcchk[7] = 1 AND a-tcchk[8] = 1 THEN a-tcchk[9] = 1.
IF a-tcchk[9] = 1 THEN DO: /* 'ws' */
  ASSIGN
    a-tcchk[7] = 1
    a-tcchk[8] = 1.
END.
 
/* check if any required terminal capabilities are missing */
/* and build m-tcchk string with them to display later          */
REPEAT ctr = 1 to 15:
  IF a-tcchk[ctr] = 0 then m-tcchk = m-tcchk + entry(ctr,c-tcchk) + " ".
END.

/* print warning message if at least one is missing */
IF m-tcchk <> "" THEN
  MESSAGE 
    "This protermcap entry missing at least one" SKIP
    "required terminal capability. These are:" SKIP m-tcchk 
    VIEW-AS ALERT-BOX WARNING BUTTONS OK.
 
FOR EACH KBIND BY KF:
  IF K-ESCSEQ <> "" THEN
    PUT STREAM TOUT UNFORMATTED ":" KF "(" KL ")=" K-ESCSEQ ":~\" skip.
END.

FOR EACH COLORS BY CNO:
  IF C-ESCSEQ <> "" THEN DO:
    IF FG <> "" AND BG <> "" THEN DO:
      PUT STREAM TOUT UNFORMATTED 
        ":COLOR " CNO " " FG "/" BG "=" C-ESCSEQ ":~\" skip.
      NEXT.
    END.
    IF FG = "" AND BG = "" THEN DO:
      PUT STREAM TOUT UNFORMATTED 
        ":COLOR " CNO "=" C-ESCSEQ ":~\" skip.
      NEXT.
    END.
    IF FG <> "" THEN DO:
      PUT STREAM TOUT UNFORMATTED 
        ":COLOR " CNO " " FG "=" C-ESCSEQ ":~\" skip.
      NEXT.
    END.
  END.
END.

PUT STREAM TOUT UNFORMATTED 
  ":.End Of " termtype " Protermcap entry:".
  
OUTPUT STREAM TOUT CLOSE.

/* pt-write.p - end of file */
