/* i-sax1h.p - This is the SAX Handler program
This small example uses a very simple approach to keeping 
track of where it is in the processing of the document. It uses currentPerson 
and currentNum, which are variables global to this procedure that enable the 
application to tie together the several different pieces of information 
that it gets for each element in the XML document. */ 
/* Name attribute for the current entry. App gets it during the StartElement 
callback */ 
DEF VAR currentPerson AS CHARACTER. 
/* Phone number from the current entry. App gets it during the Characters 
callback because it is the character data for the element. */ 
DEF VAR currentNum AS CHARACTER. 
/* This procedure is called when the parser finds the start tag for an element. 
For this particular XML doc, the app simply looks for Entry elements and 
digs out the ContactName attribute during the StartElement call, saving 
it in currentPerson */ 
PROCEDURE StartElement: 
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER. 
  DEFINE INPUT PARAMETER localName AS CHARACTER. 
  DEFINE INPUT PARAMETER qname AS CHARACTER. 
  DEFINE INPUT PARAMETER attributes AS HANDLE. 
  IF qName = "Entry" THEN 
    currentPerson = attributes:GET-VALUE-BY-QNAME("ContactName"). 
END. 

/* This callback gets passed the character data for an element. Note that SAX 
does not guarantee that all the characters for an element get passed in 
one call -- that's why the app has to maintain the currentNum global 
variable and append to it when handling Characters, and also why it has to 
wait for EndElement before displaying the message box. (Note also that some 
apps may need to use a MEMPTR to accumulate the character data, which may 
exceed the 32K Progress CHARACTER variable limit) */ 
PROCEDURE Characters: 
  DEFINE INPUT PARAMETER charData AS MEMPTR. 
  DEFINE INPUT PARAMETER numChars AS INTEGER. 
  /* ( Can assume that any call to Characters is for an Entry's text value, 
    because we know what the document looks like. If this weren't the case, 
    we'd have to keep track of the localName passed to the most recent call to 
    StartElement) */ 
  currentNum = currentNum + GET-STRING(charData, 1, GET-SIZE(charData)). 
END. 
/* This callback is called when the parser finds the end tag for an Element. 
Note that this app only cares about the end of an Entry element.*/ 
PROCEDURE EndElement: 
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER. 
  DEFINE INPUT PARAMETER localName AS CHARACTER. 
  DEFINE INPUT PARAMETER qName AS CHARACTER. 
  IF qName = "Entry" THEN 
  DO: 
    MESSAGE "Name: " currentPerson SKIP  
            "Phone Number: " currentNum VIEW-AS ALERT-BOX. 
    currentNum = "". 
    currentPerson = "". 
  END. 
END. 
/* note that, knowing the structure of the XML doc, the app could have done 
this in the EndElement call for the Phonelist element and could then have 
omitted EndDocument altogether. */ 
PROCEDURE EndDocument: 
  MESSAGE "All Done" VIEW-AS ALERT-BOX. 
END. 
