/*i-sax2h.p - This is the SAX Handler program */

/* so we can create new records*/ 
DEFINE TEMP-TABLE Custt LIKE Customer. 
DEFINE VARIABLE hBuf AS HANDLE. 
DEFINE VARIABLE hDBFld AS HANDLE. 
hBuf = BUFFER Custt:HANDLE. 
/* variable in which to accumulate all the text data for one element 
coming in through potentially multiple calls (per element) to the 
Characters procedure */ 
DEF VAR currentFieldValue as CHAR. 
/* simple-minded state machine  the code makes minimal use of it, 
but it could easily be used to validate the structure of the document 
in this example */ 
DEF VAR iProcessingState as INT. 
&SCOPED-DEFINE READY-TO-START 1 
&SCOPED-DEFINE GETTING-RECORDS 2 
&SCOPED-DEFINE GETTING-FIELDS 3 
&SCOPED-DEFINE DONE 4 

PROCEDURE StartDocument: 
  iProcessingState = {&READY-TO-START}. 
END. 

PROCEDURE StartElement: 
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER. 
  DEFINE INPUT PARAMETER localName AS CHARACTER. 
  DEFINE INPUT PARAMETER qName AS CHARACTER. 
  DEFINE INPUT PARAMETER attributes AS HANDLE. 
  IF qName = "Customers" THEN 
    iProcessingState = {&GETTING-RECORDS}. 
  ELSE IF qName = "Customer" THEN 
  DO: 
    /* starting a new customer, so create the record */ 
    CREATE Custt. 
     
    /*get the fields that are in the XML doc as attributes*/ 
    cust-num = INTEGER( attributes:GET-VALUE-BY-QNAME("Cust-num") ). 
    name = attributes:GET-VALUE-BY-QNAME( "Name" ). 
    iProcessingState = {&GETTING-FIELDS}. 
  END. 
  ELSE IF iProcessingState = {&GETTING-FIELDS} THEN 
  DO: 
    /* get a handle to the field whose name corresponds 
       to the element name */ 
    hDBFld = hBuf:BUFFER-FIELD(qName). 
     
    /* re-init the variable in which we accumulate the field value */ 
    currentFieldValue = "". 
  END. 
END. 

PROCEDURE Characters: 
  DEFINE INPUT PARAMETER charData AS MEMPTR. 
  DEFINE INPUT PARAMETER numChars AS INTEGER. 
   
  /* get the text value of the field (hDBFld was set to the correct field 
     in StartElement */ 
  currentFieldValue =  
      currentFieldValue + GET-STRING(charData, 1, GET-SIZE(charData)). 
END. 

PROCEDURE EndElement: 
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER. 
  DEFINE INPUT PARAMETER localName AS CHARACTER. 
  DEFINE INPUT PARAMETER qName as CHARACTER. 
  IF localName = "Customers" THEN 
    iProcessingState = {&DONE}. 
  ELSE IF localName = "Customer"THEN 
    iProcessingState = {&GETTING-RECORDS}. 
  ELSE IF iProcessingState = {&GETTING-FIELDS} THEN 
    hDBFld:BUFFER-VALUE = currentFieldValue. 
END. 

PROCEDURE EndDocument: 
  /* show that data made it by displaying temp-table */ 
  FOR EACH Custt: 
    DISPLAY custt. 
  END. 
  RUN Cleanup. 
END. 

PROCEDURE FatalError: 
  DEFINE INPUT PARAMETER errMessage AS CHARACTER. 
  /* not necessary to do anything with PRIVATE-DATA, this is 
     just an example of what you could do */ 
  SELF:PRIVATE-DATA = "FATAL". 
  RUN Cleanup. 
   
  /* RETURN ERROR in an error handler implicitly calls SELF:STOP-PARSING(), 
     sets SELF:PARSE-STATUS to SAX-PARSER-ERROR, and raises the Progress 
     ERROR condition */ 
  RETURN ERROR errMessage 
            + "(Line " + STRING(SELF:LOCATOR-LINE-NUMBER) 
            + ", Col " + STRING(SELF:LOCATOR-COLUMN-NUMBER) 
            + ")". 
END. 

/* this is not a SAX callback, it is just a local utility */ 
PROCEDURE Cleanup: 
  /* in case we've parsed previous documents */ 
  hBuf:EMPTY-TEMP-TABLE( ). 
END. 
