/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/************************************************************************************
	PROCEDURE: addweek.p

	PURPOSE:   Add/Subtract a number of weeks to a year/week 

	SYNTAX:    "RUN samples/addweek.p (INPUT in, OUPUT out)."

	REMARKS:   This code adds or subtract a given number of weeks to
                   a given Year/Wk combination

	PARAMETERS:
          INPUT:  A week-number (For example 199042)                    
          INPUT:  A number of weeks to be added (subtracted if negative) 
          OUTPUT: The resulting week 

	AUTHORS:   Judy Rothermal
	DATE:      February 1993

	LAST INSPECTED:
	INSPECTED BY:

 ************************************************************************************/
/*Code_Start*/
 
/* Assumptions:                                                     */
/* 1. Weeks start on MONDAYS                                        */
/* 2. If January 1st falls on Friday, Saturday, Sunday or Monday    */
/*    then week 1 for this year will start on the first Monday      */
/*    the same year. If not, week 1 will start on the last Monday   */
/*    previous year.                                                */
/*    (In other words: At least 4 of the seven days of week 1 for   */
/*     a given year must fall into this year).                      */


DEFINE INPUT  PARAMETER inweek  AS INT.   /* Input week , eg 199042    */
DEFINE INPUT  PARAMETER numweek AS INT.   /* Number of weeks to be     */
		      		          /* added (or subtracted if   */
				          /* numweek is negative)      */
DEFINE OUTPUT PARAMETER newweek AS INT.   /* Output week, eg 199052    */
DEFINE VARIABLE wdate AS DATE.            /* Work date */

/* Calculate first day of inweek */
RUN samples/begweek.p (INPUT inweek , OUTPUT wdate).    
wdate = wdate + 7 * numweek.
RUN samples/weeknum.p (wdate , OUTPUT newweek).






