/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/************************************************************************************
	PROCEDURE: flip.p

	PURPOSE:   Flips words in front to the end

	SYNTAX:    RUN samples/flip.p (INPUT in, OUTPUT out).

	REMARKS:   This code will move the first words to the end based on
                   the position of a semi-colon, comma, or the first space.

	PARAMETERS:
            INPUT:  character value to be flipped
            OUTPUT: Flipped value

	AUTHORS:   Judy Rothermal
	DATE:      February 1993

	LAST INSPECTED:
	INSPECTED BY:

 ************************************************************************************/ 
/*Code_Start*/
 
/*
This is a summary of the rules that "flip.i" uses to flip names.

When this routine is called, the first word in the name is moved
to the end of the name.  This works in the following way:

  The computer first searches for a semicolon ";".
  If present, the name is flipped at that point.
  If not, but a comma "," is, then the name is flipped there.
  Otherwise, it flips at the first space in the name.

Here are some examples:

  Wright, Steven         flips to    Steven Wright
  Wright Jr., Richard    flips to    Richard Wright Jr.
  Wright Steven          flips to    Steven Wright
  Wright Jr. Richard     flips to    Jr. Richard Wright (oops)

A good use for the semicolon is in the case of a corporation, when the
company name begins with a "The" but you do not want to have to find it by
typing "The ...," as in the following example:

  Greasy Spoon, Inc.; The    flips to    The Greasy Spoon, Inc.

The flipping feature allows "Doe, John" to be printed as "John Doe" on any
correspondance.  It is recommended that an additional variable be added to
the client record which indicates whether or not the name should in fact be
flipped.
*/

DEF VAR i  AS INT NO-UNDO.

DEF INPUT  PARAMETER in-name  AS CHAR FORMAT "x(30)" NO-UNDO.
DEF OUTPUT PARAMETER out-name AS CHAR FORMAT "x(30)" NO-UNDO.

  ASSIGN
    i   = INDEX(in-name,";")
    i   = (IF i > 0 THEN i ELSE INDEX(in-name,","))
    i   = (IF i > 0 THEN i ELSE INDEX(in-name," "))
    out-name = (IF i < 2 THEN in-name
           ELSE SUBSTR(in-name,i + 1) + " " + SUBSTR(in-name,1,i - 1)).

  /* peel off extra characters from name start and end */
  DO WHILE INDEX(" ,;",SUBSTR(out-name,1,1)) > 0 AND LENGTH(out-name) > 1:
    out-name = SUBSTR(out-name,2).
  END.
  DO WHILE INDEX(" ,;",SUBSTR(out-name,LENGTH(out-name),1)) > 1 AND 
     LENGTH(out-name) > 1:
    out-name = SUBSTR(out-name,1,LENGTH(out-name) - 1).
  END.




