/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/* 
  Sample test program for running a SmartDataObject directly on
  a local machine. 9/18/98
*/

 /*  This is what the temp-table looks like:
 *  DEFINE temp-table RowObject
 *   FIELD Cust-Num LIKE Customer.Cust-Num
 *   FIELD Name LIKE Customer.Name
 *   FIELD City LIKE Customer.City
 *   FIELD RowNum AS INT
 *   FIELD RowIdent AS CHAR
 *   FIELD RowMod AS CHAR.*/

DEFINE temp-table RowObject
  {CustSDO.i}
  {src/adm2/robjflds.i}.
  
/* This temp-table holds changes to be sent back to the server. */
DEFINE TEMP-TABLE RowObjUpd LIKE RowObject
  FIELD ChangedFields AS CHARACTER.
  
DEFINE VARIABLE iRowsReturned AS INTEGER.
DEFINE VARIABLE p AS HANDLE.
DEFINE VARIABLE cnt AS INTEGER.
DEFINE VARIABLE cMessages AS CHARACTER.
DEFINE VARIABLE cMessage  AS CHARACTER.
DEFINE VARIABLE cUndoIDs  AS CHARACTER.
DEFINE VARIABLE iCustNum  AS INTEGER.

FUNCTION openQuery LOGICAL IN p.
FUNCTION fetchMessages RETURNS CHAR IN p.

/* First start up the SDO, initialize it, and open the query. */
   
RUN CustSDO.w persistent SET p.
RUN initializeObject IN p.
openQuery().

/* Now ask the SDO to move up to 100 rows of data into the temp-table. */
RUN serverSendRows IN p 
    (0 /* no particular start row */, 
     "" /* or starting RowId */, 
     no /* no need to do a NEXT after getting the first row */, 
     100 /* maximum number of rows to batch*/, 
     OUTPUT iRowsReturned  /* actual number of rows returned */,
     OUTPUT TABLE RowObject).
       
/* Display the data we got back. */
MESSAGE "Number of rows fetched: " iRowsReturned VIEW-AS ALERT-BOX.     
MESSAGE "About to display rows fetched" VIEW-AS ALERT-BOX.
FOR EACH RowObject:
   DISPLAY RowObject.cust-num 
           RowObject.name format "x(20)" 
           RowObject.city format "x(12)"
           RowObject.rowmod format "x" 
           RowObject.rowident format "x(12)".
END.

/* Now modify each customer from 1 to 4: add "xx" to the city field,
   or if the field already ends with "xx", remove it. */
   
FOR EACH RowObject WHERE RowObject.cust-num < 5:
  /* Create an Update row with the unchanged copy of the record,
     but listing those fields which we are updating. This ChangedFields
     list goes into the before copy of the record.
  */
  
  CREATE RowObjUpd.
  BUFFER-COPY RowObject TO RowObjUpd
    ASSIGN RowObjUpd.RowMod = ""
           RowObjUpd.ChangedFields = "City".
           
  /* Now create an "after" copy of the record; set RowMod to "U"pdate
     and change the "city" field. */
  
  CREATE RowObjUpd.
  BUFFER-COPY RowObject TO RowObjUpd
     ASSIGN RowObjUpd.RowMod = "U".
  cnt = INDEX(RowObject.City, "xx").
  IF cnt = 0 THEN
    RowObjUpd.City = RowObject.City + "xx".
  ELSE 
    RowObjUpd.City = SUBSTR(RowObject.City, 1, cnt - 1).
END.

/* Now Add a new customer, using the next available cust-num > 100.
   Set RowMod to "A"dd to indicate that this is a new record,
   and give it a high RowNum to make sure it doesn't conflict with
   any other row in the table of data we received. */
   
FIND LAST RowObject WHERE RowObject.cust-num > 100 AND RowObject.cust-num < 1000
  NO-ERROR.
iCustNum = IF AVAILABLE RowObject THEN RowObject.cust-num + 1 ELSE 101.

CREATE RowObjUpd.
ASSIGN RowObjUpd.cust-num = iCustNum
       RowObjUpd.Name = "Cust " + STRING(iCustNum)
       RowObjUpd.City = "Bedford"
       RowObjUpd.RowMod = "A"
       RowObjUpd.RowNum = 100001
       RowObjUpd.ChangedFields = "Cust-num,Name,City".
       
/* Now Delete the first record with a cust-num > 1000, if there is one.
   Set RowMod to "D" to indicate the the row is to be deleted. */
   
FIND FIRST RowObject WHERE RowObject.cust-num > 1000 NO-ERROR.
IF AVAILABLE RowObject THEN 
DO:
  CREATE RowObjUpd.
  BUFFER-COPY RowObject TO RowObjUpd
    ASSIGN RowObjUpd.RowMod = "D".
END.

/* Now pass the changes back to be written to the database. */
MESSAGE "About to send updates" VIEW-AS ALERT-BOX.
RUN serverCommit IN p (INPUT-OUTPUT TABLE RowObjUpd, OUTPUT cMessages,
  OUTPUT cUNdoIDs).
  
/* The output parameter cMessages won't actually hold any error messages
   unless we're really running on an AppServer. So if it doesn't return
   anything we also need to use the fetchMessages function to get any
   messages. These messages are a delimited string. */
   
IF cMessages = "" THEN
  cMessages = fetchMessages().
IF cMessages NE "" THEN
DO cnt = 1 TO NUM-ENTRIES(cMessages, CHR(3)):
  cMessage = ENTRY(cnt, cMessages, CHR(3)).
  MESSAGE ENTRY(1, cMessage, CHR(4)) VIEW-AS ALERT-BOX.
END.

/* Finally, redisplay records which have been added, updated or deleted */
MESSAGE "About to show summary of added/updated/delete records" VIEW-AS ALERT-BOX.
FOR EACH RowObjUpd: 
  DISPLAY RowObjUpd.cust-num RowObjUpd.name RowObjUpd.city.
END.


