package jclient91;

import com.progress.open4gl.*;

// Demonstrates access to an SDO from Java using the SDOResultSet class.
// The CustSDO.w and CustSDO.i files are in the src/samples/open4gl/SDO/4gl
// directory
public class SDOApp1
{
        public static void main(String args[]) throws Exception
        {
             // Create an AppObject and connect to the AppServer
	     // Note that the AppObject is not generated by ProxyGen,
	     // It's part of the  com.progress.open4gl package.
             SDOAppObject appObj =
                 new SDOAppObject("AppServer://catalina:2055/asbroker1",
                                        "", "", "");
             System.out.println("Connected to the AppServer!");

             // Create an SDOResultSet for the  CustSDO.w SDO
	     // Note that ProxyGen is not required - SDOResultSet is
	     // part of the  com.progress.open4gl package.
             com.progress.open4gl.SDOResultSet rSet =
                         appObj._createSDOResultSet("CustSDO.w");

            int numRows = 0;
            int custNum = 0;
            String lastRowid = null;

	    // Batch is used for better performance.
            rSet.startBatch();

            while (rSet.next())
            {
                  // Display the record
                  custNum = rSet.getInt("Cust-Num");
                  String name = rSet.getString("Name");
                  String city = rSet.getString("City");
                  System.out.println(custNum + " " + name + " " + city);

                  // Update the first 5 rows by changing the City field
                  //  (column 3). If the field ends in "xx", this is removed,
                  // otherwise it is a added to the end.

                  if (numRows < 5)
                  {
                    if (city.endsWith("xx"))
                      city = city.substring(0, city.length() - 2);
                    else
                      city = city.concat("xx");
                    rSet.updateString("City", city);
                    // Commit the changes in the modified row.
                    rSet.updateRow();
                  }

                  lastRowid = rSet.getRowIdentity();
                  numRows++;
            }
            System.out.println("Number of records fetched  " + numRows + "\n");

            // Add a new customer
	     rSet.moveToInsertRow();
             int cNum = custNum + 1;
	     rSet.updateInt("Cust-Num", cNum);
	     rSet.updateString("Name", "Ariel" + cNum);
	     rSet.updateString("City", "Brookline");
	     rSet.insertRow();

             // Add another new customer
	     rSet.moveToInsertRow();
             cNum = custNum + 2;
	     rSet.updateInt("Cust-Num", cNum);
	     rSet.updateString("Name", "Ginger" + cNum);
	     rSet.updateString("City", "Avon");
	     rSet.insertRow();

             // Send the modifications and insert to the AppServer.
             rSet.sendBatch();

             // Re-open the query starting at last existing row. Need to make the inserted rows visible within the ResultSet
             rSet.reOpenQuery(lastRowid);

             rSet.next();  // Skip the last existing row
             // Let's print out the new rows we just created
             while (rSet.next())
            {
                  // Display the record
                  custNum = rSet.getInt("Cust-Num");
                  String name = rSet.getString("Name");
                  String city = rSet.getString("City");
                  System.out.println("New row: " + custNum + " " + name + " " + city);
            }

             // Now delete last row (we just created)
             rSet.last();

             custNum = rSet.getInt("Cust-Num");
             String name = rSet.getString("Name");
             String city = rSet.getString("City");
             System.out.println("deleting last row: " + custNum + " " + name + " " + city );
             System.out.println();

	     rSet.deleteRow();  // We're not batching delete, send immediately to AppServer

	     // Re-open the query after delete
             rSet.reOpenQuery();

            // Display the records
            while (rSet.next())
            {
                  custNum = rSet.getInt("Cust-Num");
                  name = rSet.getString("Name");
                  city = rSet.getString("City");
                  System.out.println(custNum + " " + name + " " + city);
            }

             rSet.close();
             appObj._release();

        }
}
