/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
*                                                                    *
*********************************************************************/
/*
 * June, 1997
 * Version 1.0
 */

/*
 * This program is run persistently from sessions.w. It acts as a broker for
 * all windows displaying information for a given session.
 */

define input parameter parenthdl as handle no-undo.
define input parameter sessionid as int no-undo.
define var nextval as int no-undo.
define var hdl as handle no-undo.
define temp-table ttabchild field prochdl as handle
                            field mainsrcid as int
                            field proc_uniqueid as int
                            field create_order as int
                            index srcid mainsrcid create_order.

ON CLOSE OF THIS-PROCEDURE DO:
   FOR EACH ttabchild:
      IF VALID-HANDLE(ttabchild.prochdl)
         AND ttabchild.prochdl:type EQ "PROCEDURE"
         AND ttabchild.prochdl:unique-id EQ ttabchild.proc_uniqueid
      THEN
         APPLY "CLOSE" TO ttabchild.prochdl. /* will call killchild */
      ELSE
         DELETE ttabchild.
   END.
   RUN killchild IN parenthdl (this-procedure).
   DELETE PROCEDURE THIS-PROCEDURE.
END.


PROCEDURE killchild :
  def input parameter childhdl as Handle no-undo.
  DEFINE BUFFER ttabchild for ttabchild.

  FIND FIRST ttabchild where ttabchild.prochdl = childhdl no-error.
  IF NOT AVAILABLE(ttabchild) THEN
     RETURN.
  DELETE ttabchild.
END PROCEDURE.


PROCEDURE session_summary:
  FIND FIRST ttabchild WHERE ttabchild.mainsrcid = -1 NO-ERROR.
  IF AVAILABLE(ttabchild)
     AND VALID-HANDLE(ttabchild.prochdl)
     AND ttabchild.prochdl:type EQ "PROCEDURE"
     AND ttabchild.prochdl:unique-id EQ ttabchild.proc_uniqueid
  THEN DO:
     RUN enable_ui IN ttabchild.prochdl.
     current-window = ttabchild.prochdl:current-window.
     apply "entry" to current-window.
  END.
  ELSE DO:
     IF AVAILABLE(ttabchild) THEN /* must be the handle's invalid */
        DELETE ttabchild.
     RUN viewsess.w PERSISTENT SET hdl.
     CREATE ttabchild.
     ASSIGN
        ttabchild.prochdl = hdl
        ttabchild.mainsrcid = -1
        ttabchild.proc_uniqueid = hdl:unique-id
        ttabchild.create_order = nextval
        nextval = nextval + 1.
     RUN initializeself IN hdl (INPUT this-procedure, INPUT sessionid).
     current-window = ttabchild.prochdl:current-window.
     apply "entry" to current-window.
  END.
END PROCEDURE.

procedure reuse_me:
define input parameter mainsrcid as int no-undo.
define input parameter prochdl as handle no-undo.
define input parameter newsrcid as int no-undo.
define input parameter inplineno as int no-undo.
def buffer mainsrc for source.

  FIND FIRST source WHERE source.session-id = sessionid
                      AND source.srcid = newsrcid NO-ERROR.
  IF NOT AVAILABLE(source) THEN
     RETURN.
  if source.parent eq 0 then
     find mainsrc where recid(mainsrc) = recid(source).
  else
     find mainsrc where source.session-id = mainsrc.session-id
                    and source.parent = mainsrc.srcid.

  find first ttabchild where mainsrcid = ttabchild.mainsrcid 
                         and ttabchild.proc_uniqueid = prochdl:unique-id.
  ttabchild.mainsrcid = mainsrc.srcid.
  run initializeself in prochdl (this-procedure, sessionid, mainsrc.srcid).
  if source.parent ne 0 then
     run view_source in prochdl (newsrcid, inplineno).
end.

PROCEDURE view_source:
  DEFINE INPUT PARAMETER sourceid AS Int NO-UNDO.
  define input parameter inplineno as int no-undo.

  def buffer ttchild for ttabchild.

  FIND FIRST source WHERE source.session-id = sessionid
                      AND source.srcid = sourceid NO-ERROR.
  IF NOT AVAILABLE(source) THEN
     RETURN.
   
  REPEAT:
     FIND NEXT ttchild WHERE ttchild.mainsrcid
                             EQ (IF source.parent EQ 0 
                                 THEN source.srcid
                                 ELSE source.parent) NO-ERROR.
     IF NOT AVAILABLE(ttchild) THEN
        LEAVE. /* leave repeat */
     IF NOT VALID-HANDLE(ttchild.prochdl)
        OR ttchild.prochdl:type NE "PROCEDURE"
        OR ttchild.prochdl:unique-id NE ttchild.proc_uniqueid
     THEN DO:
        DELETE ttchild. /* must be invalid */
        NEXT. /* see if there's another for this sourceid */
     END.
     /* must have found a valid one */
     RUN view_source IN ttchild.prochdl (INPUT sourceid, inplineno).
     current-window = ttchild.prochdl:current-window.
     apply "entry" to current-window.
     RETURN.
  END.
  RUN dup_dotp IN THIS-PROCEDURE (INPUT (IF source.parent EQ 0
                                         THEN source.srcid
                                         ELSE source.parent),
                                  INPUT sourceid, inplineno).
END PROCEDURE.




PROCEDURE dup_dotp:
  DEFINE INPUT PARAMETER parentid AS Int NO-UNDO.
  DEFINE INPUT PARAMETER sourceid AS Int NO-UNDO.
  define input parameter inplineno as int no-undo.
  
  RUN c-dotp.w PERSISTENT SET hdl.
  CREATE ttabchild.
  ASSIGN
     ttabchild.prochdl = hdl
     ttabchild.mainsrcid = parentid
     ttabchild.proc_uniqueid = hdl:unique-id
     ttabchild.create_order = nextval
     nextval = nextval + 1.
  RUN initializeself IN hdl (INPUT THIS-PROCEDURE,
                         INPUT sessionid,
                         INPUT ttabchild.mainsrcid).
  RUN view_source IN ttabchild.prochdl (INPUT sourceid, inplineno).
  current-window = ttabchild.prochdl:current-window.
  apply "entry" to current-window.
END PROCEDURE.

procedure viewall_sessions:
current-window = parenthdl:current-window.
apply "entry" to current-window.
end.

/* tell main viewer to refresh it's browse and reposition on this spot */
procedure refreshmain:
 def input parameter sessid as int no-undo.
   run refreshmain in parenthdl (sessid).
end.
