 

 /*------------------------------------------------------------------------

    File        : HybridRealm

    Purpose     : OEBPM Hybrid Realm for Single Point of Authentication

    Syntax      : 

    Description : Singleton class that implements IHybridRealm for use as

                  a single point of authentication for OEBPM.

    Author(s)   : davec

    Created     : Tue Feb 26 10:41:36 EST 2013

    Notes       : 

  ----------------------------------------------------------------------*/



USING Progress.Lang.*.

USING Progress.Security.Realm.IHybridRealm.

USING OpenEdge.Security.Util.*.



ROUTINE-LEVEL ON ERROR UNDO, THROW.



CLASS OpenEdge.Security.Realm.HybridRealm IMPLEMENTS IHybridRealm:

    

    DEFINE PRIVATE STATIC VARIABLE attributes AS CHARACTER NO-UNDO INITIAL "lastname,firstname,phone,email,description". 

    DEFINE PRIVATE /*STATIC*/ VARIABLE debugMsg AS LOGICAL NO-UNDO INITIAL FALSE.

    DEFINE PRIVATE /*STATIC*/ VARIABLE passwd AS CHARACTER NO-UNDO.

    DEFINE PRIVATE VARIABLE role AS CHARACTER NO-UNDO.

    

    DEFINE PRIVATE VARIABLE spaProps AS CLASS OpenEdge.Security.Util.Properties.

      

	/*------------------------------------------------------------------------------

	 Purpose:

	 Notes:

	------------------------------------------------------------------------------*/

		

	CONSTRUCTOR PUBLIC HybridRealm (  ):

		SUPER ().

		spaProps = NEW Properties("spaservice.properties").

        passwd = spaProps:GetCharacterProperty("Password", "").

		role = spaProps:GetCharacterProperty("Role", "SpaClient").

		debugMsg = spaProps:GetLogicalProperty("DebugMsg", ?).

		IF debugMsg THEN DO:

		    MESSAGE "Loaded property file spaservice.properties".

		    MESSAGE "   Password: "  passwd.

		    MESSAGE "   Role: " role.

		    MESSAGE "   DebugMsg: True".

		END. 

		

	END CONSTRUCTOR.





    METHOD PUBLIC CHARACTER GetAttribute( INPUT theUserId AS INTEGER, INPUT attrName AS CHARACTER ):

        

        DEFINE VARIABLE retVal AS CHARACTER NO-UNDO.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN 

            MESSAGE "Attempting to get Attribute " attrName " for user number " theUserId.

        FIND FIRST _User WHERE _User._User_number = theUserId NO-ERROR.

        IF AVAILABLE _User THEN DO:

            CASE attrName:

                WHEN "lastname" THEN

                    retVal = _User._Surname.

                WHEN "firstname" THEN 

                    retVal = _User._Given_name.

                WHEN "phone" THEN 

                    retVal = _User._Telephone.

                WHEN "email" THEN 

                    retVal = _User._Email.

                WHEN "description" THEN 

                    retVal = _User._Description.

                WHEN "userid" THEN 

                    retVal = _User._Userid.

                OTHERWISE DO:

                    retVal = ?.

                END.

            END.

        END.

        ELSE   

            UNDO, THROW NEW Progress.Lang.AppError("Invalid User Id").



        RELEASE _User.

        IF debugMsg THEN 

            MESSAGE "Returning " retVal " for attribute value".

        RETURN retVal.

        

    END METHOD.



    METHOD PUBLIC CHARACTER EXTENT GetAttributeNames( INPUT theUserId AS INTEGER ):

        

        DEFINE VARIABLE results AS CHARACTER EXTENT NO-UNDO.

        DEFINE VARIABLE attrNames AS CHARACTER EXTENT 10 NO-UNDO.

        DEFINE VARIABLE attrVal AS CHARACTER NO-UNDO.

        DEFINE VARIABLE numAttrs AS INTEGER NO-UNDO INITIAL 0.

        DEFINE VARIABLE i AS INTEGER NO-UNDO.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN 

            MESSAGE "Getting attributes for user number " theUserId.

        FIND FIRST _User WHERE _User._User_number = theUserId NO-ERROR.

        IF AVAILABLE _User THEN DO:

            /* Check to see what attributes have values */

            attrVal = _User._Surname.

            IF attrVal <> ? THEN DO:

                 numAttrs = numAttrs + 1.

                 attrNames[numAttrs] = "lastname".

            END.

            attrVal = _User._Given_name.

            IF attrVal <> ? THEN DO:

                numAttrs = numAttrs + 1.

                 attrNames[numAttrs] = "firstname".

            END.

            attrVal = _User._Telephone.

            IF attrVal <> ? THEN DO:

                numAttrs = numAttrs + 1.

                 attrNames[numAttrs] = "phone".

            END.

            attrVal = _User._Email.

            IF attrVal <> ? THEN DO:

                numAttrs = numAttrs + 1.

                 attrNames[numAttrs] = "email".

            END.

            attrVal = _User._Description.

            IF attrVal <> ? THEN DO:

                numAttrs = numAttrs + 1.

                 attrNames[numAttrs] = "description".

            END.

            attrVal = _User._Userid.

            IF attrVal <> ? THEN DO:

               numAttrs = numAttrs + 1.

                 attrNames[numAttrs] = "userid".

            END.



            IF numAttrs > 0 THEN DO:

                IF debugMsg THEN 

                    MESSAGE "Number of attribute names: " numAttrs.

                EXTENT(results) = numAttrs.

                DO i = 1 TO numAttrs:

                    results[i] = attrNames[i].

                    IF debugMsg THEN 

                        MESSAGE results[i].

                END.

            END.

        END.

        ELSE   

            UNDO, THROW NEW Progress.Lang.AppError("User not found").



        RELEASE _User.

        RETURN results.

        

    END METHOD.



    METHOD PUBLIC CHARACTER EXTENT GetUserNames(  ):

        

        DEFINE VARIABLE results AS CHARACTER EXTENT NO-UNDO.

        DEFINE VARIABLE num AS INTEGER NO-UNDO INITIAL 0.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

            

        /* First count the names */

        FOR EACH _User:

            num = num + 1.

        END. 

        

        EXTENT(results) = num.

        num = 1.

        FOR EACH _User:

            results[num] = _User._Userid.

            num = num + 1.

        END.

        

        RELEASE _User.

        RETURN results.



    END METHOD.



    METHOD PUBLIC CHARACTER EXTENT GetUserNamesByQuery( INPUT queryString AS CHARACTER ):

        

        DEFINE VARIABLE results AS CHARACTER EXTENT NO-UNDO.

        DEFINE VARIABLE num AS INTEGER NO-UNDO INITIAL 0.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN 

            MESSAGE "QueryString = " queryString. 

            

        /* First count the names */

        FOR EACH _User WHERE _User._Userid MATCHES queryString:

            IF debugMsg THEN 

                MESSAGE "Match: " _User._Userid.

            num = num + 1.

        END. 

        

        IF num = 0 THEN DO:

            IF debugMsg THEN 

                MESSAGE "No users found".

            RETURN results.

        END.

        

        EXTENT(results) = num.

        num = 0.

        FOR EACH _User WHERE _User._Userid MATCHES queryString:

            num = num + 1.

            results[num] = _User._Userid.

        END.

        IF debugMsg THEN 

            MESSAGE "Found " num " users".

        

        RELEASE _User.

        RETURN results.

        



    END METHOD.



    METHOD PUBLIC CHARACTER EXTENT GetUserNamesByQuery( INPUT attrName AS CHARACTER, INPUT attrValue AS CHARACTER ):

        

        DEFINE VARIABLE results AS CHARACTER EXTENT NO-UNDO.

        DEFINE VARIABLE temp AS CHARACTER EXTENT 100 NO-UNDO.

        DEFINE VARIABLE num AS INTEGER NO-UNDO INITIAL 0.

        DEFINE VARIABLE whichAttr AS INTEGER NO-UNDO.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN 

            MESSAGE "Attribute name: " attrName " Attribute value: " attrValue.

        whichAttr =  LOOKUP(attrName, attributes).

        IF whichAttr = 0 THEN RETURN results.

        

        /* First count the names */

        FOR EACH _User:

            CASE whichAttr:

                WHEN 1 THEN

                    IF _User._Surname MATCHES attrValue THEN DO:

                        IF debugMsg THEN 

                            MESSAGE "Match: " _User._Userid.

                        num = num + 1.

                        temp[num] = _User._Userid.

                    END.

                WHEN 2 THEN 

                    IF _User._Given_name MATCHES attrValue THEN DO:

                        IF debugMsg THEN 

                            MESSAGE "Match: " _User._Userid.

                        num = num + 1.

                        temp[num] = _User._Userid.

                    END.

                WHEN 3 THEN 

                    IF _User._Telephone MATCHES attrValue THEN DO:

                        IF debugMsg THEN 

                            MESSAGE "Match: " _User._Userid.

                        num = num + 1.

                        temp[num] = _User._Userid.

                    END.

                WHEN 4 THEN 

                    IF _User._Email MATCHES attrValue THEN DO:

                        IF debugMsg THEN 

                            MESSAGE "Match: " _User._Userid.

                        num = num + 1.

                        temp[num] = _User._Userid.

                    END.

                WHEN 5 THEN 

                    IF _User._Description MATCHES attrValue THEN DO:

                        IF debugMsg THEN 

                            MESSAGE "Match: " _User._Userid.

                        num = num + 1.

                        temp[num] = _User._Userid.

                    END.

            END.

        END. 

        

        IF num = 0 THEN DO:

            IF debugMsg THEN 

                MESSAGE "No users found".

            RETURN results.

        END.

        

        EXTENT(results) = num.

        DO whichAttr = 1 TO num:

            results[whichAttr] = temp[whichAttr].

        END.

        IF debugMsg THEN 

            MESSAGE "Found " num " users".

        

        RELEASE _User.

        RETURN results.

        



    END METHOD.





    METHOD PUBLIC LOGICAL RemoveAttribute( INPUT theUserId AS INTEGER, INPUT attrName AS CHARACTER ):

        

        DEFINE VARIABLE retVal AS LOGICAL NO-UNDO INITIAL TRUE.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN 

            MESSAGE "Attempting to remove Attribute " attrName " for user number " theUserId.

        FIND FIRST _User WHERE _User._User_number = theUserId NO-ERROR.

        IF AVAILABLE _User THEN DO:

            CASE attrName:

                WHEN "lastname" THEN

                    _User._Surname = ?.

                WHEN "firstname" THEN 

                    _User._Given_name = ?.

                WHEN "phone" THEN 

                    _User._Telephone = ?.

                WHEN "email" THEN 

                    _User._Email = ?.

                WHEN "description" THEN 

                    _User._Description = ?.

                OTHERWISE DO:

                    retVal = FALSE.

                    RELEASE _User.

                END.

            END.

        END.

        ELSE   

            UNDO, THROW NEW Progress.Lang.AppError("Invalid User Id").



        RELEASE _User.

        RETURN retVal.



    END METHOD.



    METHOD PUBLIC LOGICAL SetAttribute( INPUT theUserId AS INTEGER, INPUT attrName AS CHARACTER, INPUT attrValue AS CHARACTER ):

        

        DEFINE VARIABLE retVal AS LOGICAL NO-UNDO INITIAL TRUE.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN 

            MESSAGE "Attempting to set Attribute " attrName " for user number " theUserId " with a value of " attrValue.

        FIND FIRST _User WHERE _User._User_number = theUserId NO-ERROR.

        IF AVAILABLE _User THEN DO:

            CASE attrName:

                WHEN "lastname" THEN

                    _User._Surname = attrValue.

                WHEN "firstname" THEN 

                    _User._Given_name = attrValue.

                WHEN "phone" THEN 

                    _User._Telephone = attrValue.

                WHEN "email" THEN 

                    _User._Email = attrValue.

                WHEN "description" THEN 

                    _User._Description = attrValue.

                OTHERWISE DO:

                    retVal = FALSE.

                    RELEASE _User.

                END.

            END.

        END.

        ELSE   

            UNDO, THROW NEW Progress.Lang.AppError("Invalid User Id").



        RELEASE _User.

        RETURN retVal.



    END METHOD.



	/*------------------------------------------------------------------------------

	 Purpose:

	 Notes:

	------------------------------------------------------------------------------*/



	METHOD PROTECTED LOGICAL ValidateClient(  ):

		

		DEFINE VARIABLE result AS LOGICAL NO-UNDO INITIAL FALSE.

		DEFINE VARIABLE hCP AS HANDLE NO-UNDO.



        hCP = SESSION:CURRENT-REQUEST-INFO:GetClientPrincipal().

        IF (? <> hCP) THEN

            result = hCP:VALIDATE-SEAL (passwd).

        IF RESULT = TRUE THEN DO:

            IF hCP:ROLES <> role THEN 

                RESULT = FALSE.

        END.    

        

		RETURN result.



	END METHOD.



    METHOD PUBLIC LOGICAL ValidatePassword( INPUT theUserId AS INTEGER, INPUT password AS CHARACTER ):

        

        DEFINE VARIABLE retVal AS LOGICAL NO-UNDO INITIAL FALSE.      

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        IF debugMsg THEN DO:

            MESSAGE "Validate password using basic".

            MESSAGE "Password: " password.

        END.



        FIND FIRST _User WHERE _User._User_number = theUserId NO-ERROR.

        IF AVAILABLE _User THEN DO:

            IF _User._Password = password THEN 

                retVal = TRUE.

            ELSE

                RETURN FALSE.

        END.

        ELSE   

            UNDO, THROW NEW Progress.Lang.AppError("Invalid User Id").



        RELEASE _User.

        RETURN retVal.

        

    END METHOD.



    METHOD PUBLIC LOGICAL ValidatePassword( INPUT theUserId AS INTEGER, INPUT digest AS CHARACTER, INPUT nonce AS CHARACTER, INPUT timestamp AS CHARACTER ):

        

        DEFINE VARIABLE retVal AS LOGICAL NO-UNDO INITIAL FALSE.

        DEFINE VARIABLE password AS CHARACTER NO-UNDO.

        DEFINE VARIABLE myDigest AS CHARACTER NO-UNDO.

        DEFINE VARIABLE df AS CHARACTER NO-UNDO. 

        DEFINE VARIABLE remoteTS AS DATETIME-TZ NO-UNDO.

        DEFINE VARIABLE dif AS INT64 NO-UNDO.

        DEFINE VARIABLE hash AS RAW NO-UNDO.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        FIND FIRST _User WHERE _User._User_number = theUserId NO-ERROR.

        IF AVAILABLE _User THEN DO:

            password = _User._Password.

        END.

        ELSE 

          UNDO, THROW NEW Progress.Lang.AppError("User not found").



        RELEASE _User.

        

        IF debugMsg THEN DO: 

            MESSAGE "Validate password using digest".

            MESSAGE "Digest: " digest.

            MESSAGE "Nonce: " nonce.

            MESSAGE "Timestamp: " timestamp.

        END.

        

        df = SESSION:DATE-FORMAT.

        SESSION:DATE-FORMAT = "ymd".

        remoteTS = DATETIME-TZ(timestamp).

        SESSION:DATE-FORMAT = df.

        

        dif = ABSOLUTE(INTERVAL(NOW, remoteTS, "seconds")).

        

        IF dif > 1 THEN

          UNDO, THROW NEW Progress.Lang.AppError("Old timestamp sent").



        myDigest = password + nonce + timestamp.

        hash = SHA1-DIGEST(myDigest).

        myDigest = BASE64-ENCODE(hash).

        IF debugMsg THEN 

            MESSAGE "Computed digest: " myDigest.

        IF myDigest = digest THEN 

            retVal = TRUE.

        ELSE

            retVal = FALSE.

        RETURN retVal.

        

    END METHOD.



    METHOD PUBLIC INTEGER ValidateUser( INPUT userName AS CHARACTER ):

        

        DEFINE VARIABLE userNum AS INTEGER NO-UNDO INITIAL -1.

        

        IF THIS-OBJECT:ValidateClient() = FALSE THEN

            UNDO, THROW NEW Progress.Lang.AppError("Unauthorized client", 1).

        

        FIND FIRST _User WHERE _User._Userid = userName NO-ERROR.

        IF AVAILABLE _User THEN DO:

            userNum = _User._User_number.

            RELEASE _User.

        END.



        RETURN userNum.



    END METHOD.



END CLASS.