/*--------------------------------------------------------------------
  Name: httpd.p
  Purpose: Simple HTTP Daemon (web server) example for TCP Sockets in 
           Progress 4GL
  Author: Edsel Garcia
  
  This program demonstrates a very simple Web Server written in Progress
  4GL using the 4GL socket feature. It can run under both UNIX and Windows.
  The program creates a server-socket, establishes a read-procedure, and 
  then uses the read-procedure to handle requests for data. A Web Browser 
  that connects to the server will display the following links (each 
  represents a request that the program's read-procedure knows how to handle):
  
     Home Page: The server simply creates a Progress CHAR variable whose
                value is the HTML for a simple page, then puts the CHAR
                into a MEMPTR and sends it through the socket
                
     Sample Dynamic 4GL code: The server generates HTML text by using 
                4GL calls to get the date and time and to specify 
                different header levels
                
     Page in htdocs : Demonstrates how to add your own pages to the
                sample. The server uses the value of the link to
                retrieve and send a file of the same name from the
                htdocs directory, which is assumed to be a subdirectory
                of the current directory Progress starts. (The program 
                creates a directory by this name during initialization 
                if there is not one. To use this sample, however, the 
                directory must already be there with the file in it.) 
                To add your own files, put them into the htdocs directory
                and add the appropriate line in the readHandler procedure
                in this file where it sets up the menu. Note that this 
                sample as written uses the file "4GLSockets.htm", so that
                file must be in ./htdocs. 
                
     About:     Like the Home Page, this is simply a CHAR variable with
                hard-coded HTML in it.
                
     source code ("Click here"): The server embeds the program's .ped
                file in the appropriate HTML tags and sends it to the
                browser.
--------------------------------------------------------------------*/

DEFINE VARIABLE hServerSocket AS HANDLE NO-UNDO.
DEFINE VARIABLE vport AS INTEGER LABEL "TCP Port" FORMAT ">>>>9"
       INITIAL 33735 NO-UNDO.
DEFINE BUTTON b-start LABEL "Start Server".
&IF "{&OPSYS}" = "WIN32" &THEN
DEFINE BUTTON b-visit LABEL "Test WebSite".
&ENDIF
DEFINE STREAM ifile.
DEFINE VARIABLE wlog AS LOGICAL NO-UNDO.
DEFINE VARIABLE hInstance AS INTEGER NO-UNDO.

SESSION:APPL-ALERT-BOXES = YES.
FORM vport 
     b-start
&IF "{&OPSYS}" = "WIN32" &THEN    
     b-visit
&ENDIF
     WITH FRAME xx THREE-D SIDE-LABELS.

ON 'CHOOSE':U OF b-start
DO:
   ASSIGN vport.
   IF SELF:LABEL = "Start Server" THEN DO:
      CREATE SERVER-SOCKET hServerSocket.
      hServerSocket:SET-CONNECT-PROCEDURE("connProc").
      wlog = hServerSocket:ENABLE-CONNECTIONS("-S " + STRING(vport)) 
           NO-ERROR.
      IF wlog THEN b-start:LABEL = "Stop Server".
      ELSE MESSAGE "Server could not start." VIEW-AS ALERT-BOX.
   END.
   ELSE DO:
      hServerSocket:DISABLE-CONNECTIONS().
      DELETE OBJECT hServerSocket.
      b-start:LABEL = "Start Server".
   END.
   RETURN.
END.

&IF "{&OPSYS}" = "WIN32" &THEN
ON 'CHOOSE':U OF b-visit
DO:
   RUN ShellExecuteA(0, "open", "http://localhost:" + STRING(vport) 
       + "/", "", "", 0, OUTPUT hInstance).
    RETURN.
END.
&ENDIF

OS-CREATE-DIR "htdocs".
DISPLAY vport WITH FRAME xx.
ENABLE vport b-start 
&IF "{&OPSYS}" = "WIN32" &THEN
    b-visit
&ENDIF        
    WITH FRAME xx.
DISPLAY 
  "To test this server use a web browser and enter url:"
  "http://server:port/" SKIP
  "You can put files into dir htdocs to be be accessed via http" SKIP
  "The section to return the MIME Types can be edited to add" SKIP
  "support for additional types" SKIP
  "Hit " + KBLABEL("END-ERROR") + " to terminate this session" FORMAT
"x(45)".

CREATE SERVER-SOCKET hServerSocket.
hServerSocket:SET-CONNECT-PROCEDURE("connProc").
hServerSocket:ENABLE-CONNECTIONS("-S " + STRING(vport)).
b-start:LABEL = "Stop Server".
WAIT-FOR WINDOW-CLOSE OF CURRENT-WINDOW.
hServerSocket:DISABLE-CONNECTIONS().
DELETE OBJECT hServerSocket.

PROCEDURE connProc.
  DEFINE INPUT PARAMETER hSocket AS HANDLE.  /*Socket implicitly
created*/
  hSocket:SET-READ-RESPONSE-PROCEDURE ("readHandler", THIS-PROCEDURE).
END.

FUNCTION socketRead RETURNS CHARACTER (h AS HANDLE, l AS INTEGER):
   DEFINE VARIABLE b AS MEMPTR NO-UNDO.
   DEFINE VARIABLE s AS CHARACTER NO-UNDO.

   SET-SIZE(b) = l + 1.
   h:READ(b, 1, l, 1).
   s = GET-STRING(b,1).
   SET-SIZE(b) = 0.
   RETURN s.
END.

PROCEDURE readHandler:
   DEFINE VARIABLE l AS INTEGER NO-UNDO.
   DEFINE VARIABLE str AS CHARACTER NO-UNDO.
   DEFINE VARIABLE mBuffer AS MEMPTR NO-UNDO.
   DEFINE VARIABLE wrequest AS CHARACTER NO-UNDO.
   DEFINE VARIABLE wdata AS CHARACTER NO-UNDO.

   l = SELF:GET-BYTES-AVAILABLE().   
   IF l > 0 THEN DO:
      str = socketRead(SELF:HANDLE, l).
      wrequest = ENTRY(1,str,CHR(10)).
      SET-SIZE(mBuffer) = 1024.
      wdata = "HTTP/1.0 200 OK~nServer: 4GL demo~nConnection: " +
              "close~nContent-type: text/html~n~n".
      wdata = wdata + '<A HREF="/">Home Page</A> | <A ' +
              'HREF="/4gl.html">Sample Dynamic 4GL code</A> | <A ' +
              'HREF="/4GLSockets.htm">Page in htdocs</A> | <A ' +
              'HREF="/about.html">About</A><HR>'.
      IF ENTRY(1, wrequest, " ") = "GET" THEN DO:
         CASE ENTRY(2, wrequest, " "):
         WHEN "/" THEN 
         wdata = wdata
               + "Welcome to the Simple 4GL WebServer Example"
               + "<BR><BR><BR>"
               + 'Click <A HREF="/source.txt">here</A> to see the '
               + 'source code of this program. ({&FILE-NAME})'.
         WHEN "/4gl.html" THEN RUN 4gltest (INPUT-OUTPUT wdata).
         WHEN "/about.html" THEN 
           wdata = wdata + "The Simple 4GL Web Server Example was " 
               + "written by Edsel Garcia at PSC Technical Suport"
               + "~n<BR>This program is intended to be an example "
               + "of using Progress 4GL server sockets.".
         WHEN "/source.txt" THEN DO:
                  wdata = "".
                  RUN getfile ("{&FILE-NAME}").
             END.
         OTHERWISE DO:
                  wdata = "".
                  RUN getfile ("htdocs" + ENTRY(2, wrequest, " ")).
             END.
         END CASE.
      END.
      ELSE DO:
         wdata = wdata + "Request: " 
               + wrequest + "~nRequest not implemented.".
      END.
      IF wdata <> "" THEN DO:
          PUT-STRING(mBuffer,1) = wdata.
          SELF:WRITE (mBuffer,1,LENGTH(wdata)).
          SET-SIZE(mBuffer) = 0.
      END.
      SELF:DISCONNECT().
   END.
   ELSE DO:
      SELF:DISCONNECT().
   END.
END.

PROCEDURE 4gltest:
DEFINE INPUT-OUTPUT PARAMETER wdata AS CHARACTER NO-UNDO.
DEFINE VARIABLE I AS INTEGER NO-UNDO.

wdata = wdata + "Today's date: " + STRING(TODAY) + " Time: " +
STRING(TIME,"HH:MM:SS") + "<BR>".
DO I = 1 TO 6:
   wdata = wdata 
         + "<H" + STRING(I) + ">" 
         + "Header: " + STRING(I) + "</" +
STRING(I) + ">".
END.
END.

PROCEDURE getsource:
DEFINE VARIABLE wdata AS CHARACTER NO-UNDO.
DEFINE VARIABLE mBuffer AS MEMPTR NO-UNDO.
DEFINE VARIABLE wline AS CHARACTER NO-UNDO.

SET-SIZE(mBuffer) = 1024.
wdata = "HTTP/1.0 200 OK~nServer: 4GL demo~nConnection: "
      + "close~nContent-type: text/plain~n~n".
PUT-STRING(mBuffer,1) = wdata.
SELF:WRITE (mBuffer,1,LENGTH(wdata)).
FILE-INFO:FILE-NAME = "{&FILE-NAME}".
IF FILE-INFO:FULL-PATHNAME = ? THEN DO:
   wdata = "Source file not found.".
   PUT-STRING(mBuffer,1) = wdata.
   SELF:WRITE (mBuffer,1,LENGTH(wdata)).
   RETURN.
END.
INPUT STREAM ifile FROM VALUE(FILE-INFO:FULL-PATHNAME).
REPEAT:
  IMPORT STREAM ifile UNFORMATTED wline.
  wdata = wline + "~n".
  PUT-STRING(mBuffer,1) = wdata.
  SELF:WRITE (mBuffer,1,LENGTH(wdata)).
END.
INPUT STREAM ifile CLOSE.
END.

PROCEDURE getfile:
DEFINE INPUT PARAMETER pfilename AS CHARACTER NO-UNDO.
DEFINE VARIABLE wdata AS CHARACTER NO-UNDO.
DEFINE VARIABLE mBuffer AS MEMPTR NO-UNDO.
DEFINE VARIABLE vdata AS RAW NO-UNDO.
DEFINE VARIABLE wcontent AS CHARACTER NO-UNDO.

/* MIME Types definition */
CASE ENTRY(2, pfilename, "."):
    WHEN "txt" OR WHEN "ped" OR WHEN "p" THEN wcontent = "text/plain".
    WHEN "html" OR WHEN "htm" THEN wcontent = "text/html".
    WHEN "gif" THEN wcontent = "image/gif".
    WHEN "jpg" THEN wcontent = "image/jpeg".
    OTHERWISE wcontent = "*/*".
END CASE.
SET-SIZE(mBuffer) = 4096.
FILE-INFO:FILE-NAME = pfilename.

wdata = "HTTP/1.0 200 OK~nServer: 4GL demo~nConnection: "
      + "close~nContent-type: " 
      + wcontent 
/*
      + "~n"
      + "Accept-Ranges: bytes~n"
      + "Content-Length: " + STRING(FILE-INFO:FILE-SIZE) + "~n"
      + "Connection: close"
*/      
      + "~n~n".
IF FILE-INFO:FULL-PATHNAME = ? THEN DO:
   wdata = "HTTP/1.0 404 Not found~nServer: 4GL demo~nConnection: "
   + "close~nContent-type: text/html~n~n" + "Sorry page not found.".
   PUT-STRING(mBuffer,1) = wdata.
   SELF:WRITE (mBuffer,1,LENGTH(wdata)).
   RETURN.
END.
PUT-STRING(mBuffer,1) = wdata.
SELF:WRITE (mBuffer,1,LENGTH(wdata)).
INPUT STREAM ifile FROM VALUE(FILE-INFO:FULL-PATHNAME) BINARY.
LENGTH(vdata) = 4096.
REPEAT:
  IMPORT STREAM ifile UNFORMATTED vdata.
  PUT-BYTES(mBuffer,1) = vdata.
  SELF:WRITE (mBuffer,1,LENGTH(vdata)).
END.
LENGTH(vdata) = 0.
INPUT STREAM ifile CLOSE.
SET-SIZE(mBuffer) = 0.
END.

&IF "{&OPSYS}" = "WIN32" &THEN
PROCEDURE ShellExecuteA EXTERNAL "shell32.dll":
  DEFINE INPUT  PARAMETER hwnd         AS LONG.
         /* Handle to parent window */
  DEFINE INPUT  PARAMETER lpOperation  AS CHAR.
         /* Operation to perform: open, print */
  DEFINE INPUT  PARAMETER lpFile       AS CHAR.
         /* Document or executable name */
  DEFINE INPUT  PARAMETER lpParameters AS CHAR.
         /* Command line parameters to executable in lpFile */
  DEFINE INPUT  PARAMETER lpDirectory  AS CHAR.
         /* Default directory */
  DEFINE INPUT  PARAMETER nShowCmd     AS LONG.
         /* whether shown when opened: 
            0 hidden, 1 normal, minimized 2, maximized 3, 
            0 if lpFile is a document */
  DEFINE RETURN PARAMETER hInstance    AS LONG.
         /* Less than or equal to 32 */
END.
&ENDIF
