/*
  This example demonstrates how to use the SAX-READER support in the
  4GL SonicMQ Adapter.  It will read an XML message from a Sonic queue and
  write it to a LONGCHAR.
*/
DEFINE VARIABLE sessionH AS HANDLE NO-UNDO.
DEFINE VARIABLE msgConsumer AS HANDLE NO-UNDO.
DEFINE VARIABLE mesgH AS HANDLE NO-UNDO.
DEFINE VARIABLE hdl2 AS HANDLE NO-UNDO.
DEFINE VARIABLE stillWaiting AS LOGICAL INIT yes.
DEFINE VARIABLE lch AS LONGCHAR NO-UNDO view-as editor size 70 by 30 large.

/* Start up the session to the SonicMQ broker */
RUN jms/ptpsession.p PERSISTENT SET sessionH ("-H localhost -S 5162 ").
RUN setBrokerURL IN sessionH ("localhost:2506").
RUN beginSession IN sessionH.

/* Create the message consumer and start receiving messages. */
RUN createMessageConsumer IN sessionH (THIS-PROCEDURE,
                                       "messageHandler",
                                       OUTPUT msgConsumer).
RUN receiveFromQueue IN sessionH ("SampleQ1", ?, msgConsumer).
RUN startReceiveMessages IN sessionH.

/* Wait for all messages to be received. */
RUN waitForMessages IN sessionH ("inWait", THIS-PROCEDURE, ?).
RUN deleteSession IN sessionH.


/* Message handler procedure */
PROCEDURE messageHandler:
DEFINE INPUT PARAMETER messageH AS HANDLE NO-UNDO.
DEFINE INPUT PARAMETER messageConsumerH AS HANDLE NO-UNDO.
DEFINE OUTPUT PARAMETER autoReplyH AS HANDLE NO-UNDO.

DEFINE VARIABLE hdl1 AS HANDLE NO-UNDO.

CREATE SAX-WRITER hdl2.
hdl2:SET-OUTPUT-DESTINATION("LONGCHAR",lch).
hdl2:START-DOCUMENT().

CREATE SAX-READER hdl1.


/*
  The Adapter procedure SetSaxReader will set the input source
  for a SAX-READER to the XML message that has been received.
  The application may then use normal SAX-READER calls to access
  the XML from the message.
*/
RUN SetSaxReader IN messageH (hdl1).



hdl1:SAX-PARSE().
DELETE OBJECT hdl1.
RUN deleteMessage IN messageH.

hdl2:END-DOCUMENT().
DISPLAY lch.
lch = "".
stillWaiting = FALSE.
END.


FUNCTION inWait RETURNS LOGICAL.
RETURN stillWaiting.
END.


/*****************************************************/
/* callbacks for the SAX-READER function SAX-PARSE() */
/*****************************************************/

/*
   This procedure is called when the parser finds the
   start tag for an element.
*/

PROCEDURE StartElement:

DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
DEFINE INPUT PARAMETER localName AS CHARACTER.
DEFINE INPUT PARAMETER qname AS CHARACTER.
DEFINE INPUT PARAMETER attributes AS HANDLE.

hdl2:START-ELEMENT(localName, namespaceURI).

END.



/* This callback gets passed the character data for an element.*/

PROCEDURE Characters:
DEFINE INPUT PARAMETER charData AS MEMPTR.
DEFINE INPUT PARAMETER numChars AS INTEGER.

DEFINE VARIABLE data AS CHARACTER NO-UNDO.

data = GET-STRING(charData, 1, GET-SIZE(charData)).
hdl2:WRITE-CHARACTERS(data).

END.


/* This callback is called when the parser finds the end tag for an Element.*/

PROCEDURE EndElement:
DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
DEFINE INPUT PARAMETER localName AS CHARACTER.
DEFINE INPUT PARAMETER qName AS CHARACTER.

hdl2:END-ELEMENT(localName, namespaceURI).

END.



