/*
  This example demonstrates how to use the SAX-WRITER support in the
  4GL SonicMQ Adapter.  It will read in the XML from a file using the
  SAX-READER and send it to a Sonic queue using an XML message.
*/

DEFINE VARIABLE sessionH AS HANDLE NO-UNDO.
DEFINE VARIABLE mesgH AS HANDLE NO-UNDO.
DEFINE VARIABLE hdl1 AS HANDLE NO-UNDO.
DEFINE VARIABLE hdl2 AS HANDLE NO-UNDO.

/* Start up the session to the SonicMQ broker */
RUN jms/ptpsession.p PERSISTENT SET sessionH ("-H localhost -S 5162 ").
RUN setBrokerURL IN sessionH ("localhost:2506").
RUN beginSession IN sessionH.

/* Create the Sonic XML message */
RUN createXMLMessage IN sessionH (OUTPUT mesgH).

/*
   The Adapter function GetSaxWriter will return a handle to a
   newly created SAX-WRITER and set its output to a destination local
   to the Adapter.  The application may then make normal SAX-WRITER
   calls using this handle.
*/
hdl2 = DYNAMIC-FUNCTION('GetSaxWriter':u IN mesgH, ?).

hdl2:START-DOCUMENT().

/*
   Create a SAX-READER to read in the xml file to be sent in a
   SonicMQ XML message.  The SAX-READER callback procedures will
   use the handle to the SAX-WRITER just created to copy the XML.
*/
CREATE SAX-READER hdl1.
hdl1:SET-INPUT-SOURCE("FILE", "personal.xml").
hdl1:SAX-PARSE().
DELETE OBJECT hdl1.

hdl2:END-DOCUMENT().

/*
   The Adapter procedure DeleteSaxWriter will copy the XML written with
   the SAX-WRITER calls into the SonicMQ XML message.  It will then delete
   the SAX-WRITER specified by the handle.
*/
RUN DeleteSaxWriter IN mesgH (hdl2).


/* Send the XML message. */
RUN sendToQueue IN sessionH ("SampleQ1", mesgH, ?, ?, ?).

/* Disconnect the session from the SonicMQ broker */
RUN deleteMessage IN mesgH.
RUN deleteSession IN sessionH.




/*****************************************************/
/* callbacks for the SAX-READER function SAX-PARSE() */
/*****************************************************/

/* This procedure is called when the parser finds
   the start tag for an element. */

PROCEDURE StartElement:

DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
DEFINE INPUT PARAMETER localName AS CHARACTER.
DEFINE INPUT PARAMETER qname AS CHARACTER.
DEFINE INPUT PARAMETER attributes AS HANDLE.

hdl2:START-ELEMENT(localName, namespaceURI).

END.



/* This callback gets passed the character data for an element. */

PROCEDURE Characters:
DEFINE INPUT PARAMETER charData AS MEMPTR.
DEFINE INPUT PARAMETER numChars AS INTEGER.

DEFINE VARIABLE data AS CHARACTER NO-UNDO.

data = GET-STRING(charData, 1, GET-SIZE(charData)).
hdl2:WRITE-CHARACTERS(data).

END.


/* This callback is called when the parser finds the end tag for an Element. */

PROCEDURE EndElement:
DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
DEFINE INPUT PARAMETER localName AS CHARACTER.
DEFINE INPUT PARAMETER qName AS CHARACTER.

hdl2:END-ELEMENT(localName, namespaceURI).

END.

