/* This example implements a server who gets requests from a JMS queue, executes
   the request, and replies to the requester. Run several instances of this
   server and several instances of a client (example20) to observe the 
   scalability of this configuration. */

DEFINE INPUT PARAMETER serverName AS CHAR.

DEFINE VARIABLE ptpsession AS HANDLE.
DEFINE VARIABLE consumerH AS HANDLE.
DEFINE VARIABLE replyMessage AS HANDLE.

/* Creates a session object. */
RUN  jms/ptpsession.p PERSISTENT SET ptpsession ("-H localhost -S 5162 ").
RUN setBrokerURL IN ptpsession ("localhost:2506").
RUN beginSession IN ptpsession.

/* Uses one message for all the replies. */
RUN createTextMessage IN ptpsession (OUTPUT replyMessage).

/* receives requests from  the requestQueue */
RUN createMessageConsumer IN ptpsession (
                              THIS-PROCEDURE,    /* This proc will handle it */
                             "requestHandler",  /* name of internal procedure */
                              OUTPUT consumerH).

RUN receiveFromQueue IN ptpsession ("requestQueue",   /* request queue */
                                     ?,               /* No message selector */
                                     consumerH).      /* Handles the messages */

/* Start receiving requests */
RUN startReceiveMessages IN ptpsession.

/* Process requests forever. */
RUN waitForMessages IN ptpsession ("inWait", THIS-PROCEDURE, ?).

PROCEDURE requestHandler:
DEFINE INPUT PARAMETER requestH AS HANDLE.
DEFINE INPUT PARAMETER msgConsumerH AS HANDLE.
DEFINE OUTPUT PARAMETER replyH AS HANDLE.

DEFINE VAR replyText AS CHAR.


/* Creates a reply message. The reply is sent automatically when control
   returns to the 4GL-To-JMS implementation.
*/
replyText=serverName + " executed " + DYNAMIC-FUNCTION('getText':U IN requestH).
RUN deleteMessage IN requestH.
replyH = replyMessage.
RUN setText IN replyH (replyText).

END.

FUNCTION inWait RETURNS LOGICAL.
RETURN true.
END.

