/* Subscribes and receives three messages in a single transaction. */
DEFINE VARIABLE pubsubsession AS HANDLE.
DEFINE VARIABLE consumerH AS HANDLE.
DEFINE VARIABLE msgNum AS INT INIT 0.

/* Creates a transcated for receiving session. */
RUN  jms/pubsubsession.p PERSISTENT SET pubsubsession ("-H localhost -S 5162 ").
RUN setBrokerURL IN pubsubsession ("localhost:2506").
RUN setTransactedReceive IN pubsubsession.
RUN beginSession IN pubsubsession.

/* Subscribe to the TestTopic topic. Messages are handled by the
   "msgHandler" internal procedure.
*/
RUN createMessageConsumer IN pubsubsession (
                              THIS-PROCEDURE,   /* This proc will handle it */
                             "msgHandler",      /* name of internal procedure */
                              OUTPUT consumerH).

RUN subscribe IN pubsubsession ("TestTopic",  /* name of topic */
                             ?,               /* Subscription is not durable */
                             ?,               /* No message selector */
                             no,              /* Want my own messages too */
                             consumerH).      /* Handles the incoming messages*/

/* Start receiving messages */
RUN startReceiveMessages IN pubsubsession.

/* Wait to receive the three messages. */
WAIT-FOR u1 OF THIS-PROCEDURE.

PROCEDURE msgHandler:
DEFINE INPUT PARAMETER messageH AS HANDLE. 
DEFINE INPUT PARAMETER msgConsumerH AS HANDLE.
DEFINE OUTPUT PARAMETER replyH AS HANDLE.

/* Display the message - we assume that reply is not required. */
DISPLAY "Message text: " DYNAMIC-FUNCTION('getText':U IN messageH)
  FORMAT "x(70)".
RUN deleteMessage IN messageH.

msgNum = msgNum + 1.

/* Commit the reception of the three messages. */
IF msgNum = 3 THEN
DO:
  RUN commitReceive IN pubsubsession.
  message "committed!".
  APPLY "U1" TO THIS-PROCEDURE. 
END.

END.
