/* JMSgateway.p: A gateway between local and remote PUB/SUB events. */
DEFINE INPUT PARAM connectionParams AS CHAR.

/* JMS  objects */
DEFINE VARIABLE pubsubsession AS HANDLE.
DEFINE VARIABLE outMessage AS HANDLE.
DEFINE VARIABLE consumer AS HANDLE.

/* Raw Transfer Declarations */
DEFINE TEMP-TABLE rawtt FIELD val AS RAW.
CREATE rawtt.
FUNCTION bufferToRaw RETURNS RAW (bufferH AS HANDLE) FORWARD.

/* Initializes the JMS server and subscribes to the CustUpdate topic */
RUN  jms/pubsubsession.p PERSISTENT SET pubsubsession (connectionParams).
RUN setBrokerURL IN pubsubsession ("localhost:2506").
RUN beginSession IN pubsubsession.
RUN createMapMessage IN pubsubsession (OUTPUT outMessage).
RUN createMessageConsumer IN pubsubsession (THIS-PROCEDURE,
                                             "handleRemoteEvent",
                                              OUTPUT consumer).
RUN subscribe IN pubsubsession ("CustUpdate", /* Topic name */
                                ?, /* Not durable. */
                                ?, /* No message selector */
                                true, /* No local events please */
                                consumer).
RUN startReceiveMessages IN pubsubsession.

/* Subscribes to local CustUpdate events */
SUBSCRIBE TO "CustUpdate"  ANYWHERE RUN-PROCEDURE "handleLocalEvent".

/* Publish locally a remote message from the CustUpdate topic. */
PROCEDURE handleRemoteEvent:
DEFINE INPUT PARAMETER messageH AS HANDLE NO-UNDO.
DEFINE INPUT PARAMETER messageConsumerH AS HANDLE NO-UNDO.
DEFINE OUTPUT PARAMETER replyH AS HANDLE NO-UNDO.

  PUBLISH "CustUpdate"
          (DATE (DYNAMIC-FUNCTION('getChar':U IN messageH, "updateDate")),
          DYNAMIC-FUNCTION('getInt':U IN messageH, "custNum"),
          DYNAMIC-FUNCTION('getBytesToRaw':U IN messageH, "rawCust")).
  RUN deleteMessage IN messageH.
END.

/* Publish remotely a local CustUpdate event. */
PROCEDURE handleLocalEvent:
DEFINE INPUT PARAMETER dt AS DATE.
DEFINE INPUT PARAMETER custNum AS INT.
DEFINE INPUT PARAMETER custBuffer AS HANDLE.

RUN setString IN outMessage ("updateDate", STRING(dt)).
RUN setInt IN outMessage ("custNum", custNum).
RUN setBytesFromRaw IN outMessage ("rawCust", bufferToRaw(custBuffer)).
RUN publish IN pubsubsession ("CustUpdate", outMessage,  ?, ?, ?).
END.

PROCEDURE deleteGateway:
RUN deleteMessage IN outMessage.
RUN deleteSession IN pubsubsession.
DELETE OBJECT THIS-PROCEDURE.
END.

FUNCTION bufferToRaw RETURNS RAW (bufferH AS HANDLE):
/* Raw Transfer Variables */
DEFINE VAR rawbuf AS HANDLE.
DEFINE VAR rawCust AS HANDLE.

rawbuf = BUFFER rawtt:HANDLE.
rawCust = rawbuf:buffer-field(1).
bufferH:raw-transfer(true, rawCust).
RETURN rawCust:BUFFER-VALUE.
END.


