/* customers.p: Manages customer records of a specified country and keeps the other records identical to the master copy. */
DEFINE TEMP-TABLE custt LIKE customer.
DEFINE BUFFER custtUpd FOR custt.
DEFINE VAR custtH AS HANDLE.
DEFINE VAR bufferH AS HANDLE.

/* Getting a handle to a dynamic buffer.             */
custtH = TEMP-TABLE custt:HANDLE.
bufferH = custtH:DEFAULT-BUFFER-HANDLE.

/* Subscribes to CustUpdate events.                   */
SUBSCRIBE TO "CustUpdate"  ANYWHERE RUN-PROCEDURE "updateCustFromRaw".

PROCEDURE loadCustomers:
  FOR EACH customer:
      CREATE custt.
      buffer-copy customer to custt.
  END.
END.

/* Updates a customer from the "correct" country, displays customers from other
   countries. */
PROCEDURE updateCustInteractive.
  DEFINE INPUT PARAM custNum AS INT.
  DEFINE INPUT PARAM custCountry AS CHAR.
  FIND custt WHERE custt.cust-num = custNum.
  IF custt.country = custCountry THEN
  DO:
    UPDATE custt WITH 2 COL.
    PUBLISH "CustUpdate" (TODAY, custNum,  bufferH). 
  END.
  ELSE
    DISPLAY custt WITH 2 COL.
END.

/* Updates a customer record from a RAW value.   */
PROCEDURE updateCustFromRaw:
  DEFINE INPUT PARAM dt AS DATE.
  DEFINE INPUT PARAM custNum AS INT.
  DEFINE INPUT PARAM rawCust AS RAW.

  FIND custtUpd WHERE custtUpd.cust-num = custNum.
  RAW-TRANSFER rawCust TO custtUpd.
  MESSAGE custNum VIEW-AS ALERT-BOX TITLE "customer updated".
END.


