/*  DatatypeFormats.p - This code gives examples of the formats that are
    required by SOAP for the following XML Schema datatypes.  
    boolean    - true, false, 1, 0
    datetime   - 9999-99-99THH:MM:SS.SSS+HH:MM
    duration   - P99Y99M99DT99H99M99S
    gDay       - ---DD
    gMonth     - --MM--
    gMonthDay  - --MM-DD
    gYear      - YYYY
    gYearMonth - YYYY-MM
*/

DEFINE VARIABLE hWS AS HANDLE.
DEFINE VARIABLE hPortType AS HANDLE.
DEFINE VARIABLE err AS LOGICAL.
DEFINE VARIABLE ret AS LOGICAL.
DEFINE VARIABLE lcInput AS LONGCHAR VIEW-AS EDITOR LARGE SIZE 40 BY 10.
DEFINE VARIABLE lcOutput AS LONGCHAR VIEW-AS EDITOR LARGE SIZE 70 BY 10.
DEFINE VARIABLE i AS INTEGER.
DEFINE VARIABLE dtzDateIn as DATETIME-TZ.
DEFINE VARIABLE lBoolIn as LOGICAL.
DEFINE VARIABLE sdurationIn AS CHARACTER.
DEFINE VARIABLE sgDayIn AS CHARACTER.
DEFINE VARIABLE sgMonthIn AS CHARACTER.
DEFINE VARIABLE sgMonthDayIn AS CHARACTER.
DEFINE VARIABLE sgYearIn AS CHARACTER.
DEFINE VARIABLE sgYearMonthIn AS CHARACTER.

DEFINE VARIABLE dtzDateOut as DATETIME-TZ.
DEFINE VARIABLE lBoolOut as LOGICAL.
DEFINE VARIABLE sdurationOut AS CHARACTER.
DEFINE VARIABLE sgDayOut AS CHARACTER.
DEFINE VARIABLE sgMonthOut AS CHARACTER.
DEFINE VARIABLE sgMonthDayOut AS CHARACTER.
DEFINE VARIABLE sgYearOut AS CHARACTER.
DEFINE VARIABLE sgYearMonthOut AS CHARACTER.

DEFINE VARIABLE hDoc AS HANDLE.
DEFINE VARIABLE hRoot AS HANDLE.
DEFINE VARIABLE hText AS HANDLE.
DEFINE VARIABLE hNode AS HANDLE.
DEFINE VARIABLE hChildNode AS HANDLE.

CREATE SERVER hWS.  
hWS:CONNECT("-WSDL http://localhost/DatatypeFormats/Service1.asmx?WSDL ").
IF NOT hws:CONNECTED() THEN 
DO:
    MESSAGE "SERVER:CONNECTED  : " hws:CONNECTED() VIEW-AS ALERT-BOX.
    LEAVE.
END.

RUN Service1Soap SET hPortType ON SERVER hWS.
IF NOT VALID-HANDLE(hPortType) THEN 
DO:
    MESSAGE "hPortType valid: " VALID-HANDLE(hPortType) VIEW-AS ALERT-BOX.
    LEAVE.
END.

CREATE X-DOCUMENT hDoc.
CREATE X-NODEREF hRoot.
CREATE X-NODEREF hText.
CREATE X-NODEREF hNode.
CREATE X-NODEREF hChildNode.

hDoc:CREATE-NODE-NAMESPACE(hRoot, "http://tempuri.org/", "s0:MultiDatatypes", "element").
hRoot:SET-ATTRIBUTE("xmlns:s0", "http://tempuri.org/").
hDoc:APPEND-CHILD(hRoot).

/* paramIn1 boolean                                                  */
/* boolean value must one of the following values: true, false, 1, 0 */
lBoolIn = TRUE. 
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/",  "s0:paramIn1", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
hText:NODE-VALUE = STRING(lBoolIn, "true/false").
hNode:APPEND-CHILD(hText). 

/* paramIn2 datetime                                              */
/* datetime must be in the format yyyy-mm-dd T hh:mm:ss.sss+hh:mm */
/* For example, 2003-09-08T14:08:42.000-04:00                     */
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn2", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
dtzDateIn = NOW.
hText:NODE-VALUE = ISO-DATE(dtzDateIn). 
hNode:APPEND-CHILD(hText).

/*  paramIn3 duration                                                    */
/*  duration must be in the format P99Y99M99DT99H99M99S                  */
/*  For example, "P5Y6M10DT2H30M45S" 5 years, 6 months, 10 days, 2 hours */
/*  30 minutes and 45 seconds                                            */
sdurationIn = "050610023045". 
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn3", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
/* hText:NODE-VALUE = "P5Y6M10DT2H30M45S". */
hText:NODE-VALUE = STRING(sdurationIn, "P99Y99M99DT99H99M99S"). 
hNode:APPEND-CHILD(hText). 

/* paramIn4 gDay                       */
/* dDay must be in the format "---DD"  */
sgDayIn = "18".
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn4", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
hText:NODE-VALUE = STRING(sgDayIn, "---99").
hNode:APPEND-CHILD(hText). 

/* paramIn5 gMonth                        */
/* dMonth must be in the format "--MM--"  */
sgMonthIn = "05".
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn5", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
hText:NODE-VALUE = STRING(sgMonthIn, "--99--").
hNode:APPEND-CHILD(hText). 

/* paramIn6 gDay                              */
/* dMonthDay must be in the format "--MM-DD"  */
sgMonthDayIn = "0608".
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn6", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
hText:NODE-VALUE = STRING(sgMonthDayIn, "--99-99").
hNode:APPEND-CHILD(hText). 

/* paramIn7 gYear                      */
/* dYear must be in the format "YYYY"  */
sgYearIn = "2000".
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn7", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
hText:NODE-VALUE = STRING(sgYearIn, "9999").
hNode:APPEND-CHILD(hText). 

/* paramIn8 gYearMonth                         */
/* dYearMonth must be in the format "YYYY-MM" */
sgYearMonthIn = "199902".
hDoc:CREATE-NODE-NAMESPACE(hNode, "http://tempuri.org/", "s0:paramIn8", "element").
hRoot:APPEND-CHILD(hNode).
hDoc:CREATE-NODE(hText, ?, "text").
hText:NODE-VALUE = STRING(sgYearMonthIn, "9999-99").
hNode:APPEND-CHILD(hText). 

ret = hDoc:SAVE( "longchar" , lcInput).
IF ret THEN
DO:
  RUN MultiDatatypes IN hPortType (INPUT lcInput, OUTPUT lcOutput) NO-ERROR.
  RUN ErrorInfo (OUTPUT err).
  IF NOT err THEN
  DO:
    ret = hDoc:LOAD("longchar" , lcOutput, FALSE).
    hDoc:GET-DOCUMENT-ELEMENT(hRoot). 
    DO i = 1 TO hRoot:NUM-CHILDREN:
    hRoot:get-child(hNode, i).
    IF hNode:SUBTYPE = "ELEMENT" THEN
    DO:
        IF hNode:NAME = "paramOut1" THEN /* boolean */
        DO:
            Ret = hNode:GET-CHILD(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
               lBoolOut  = LOGICAL(hChildNode:NODE-VALUE).
        END.

        IF hNode:NAME = "paramOut2" THEN /* datetime */
        DO:
            SESSION:DATE-FORMAT = "ymd".
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                dtzDateOut  = DATETIME-TZ(hChildNode:NODE-VALUE).
            SESSION:DATE-FORMAT = "mdy".  /* back to original */
        END.

        IF hNode:NAME = "paramOut3" THEN /* duration */
        DO:
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                sdurationOut  = hChildNode:NODE-VALUE.
        END.

        IF hNode:NAME = "paramOut4" THEN /* gDay */
        DO:
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                sgDayOut  = hChildNode:NODE-VALUE.
        END.

        IF hNode:NAME = "paramOut5" THEN /* gMonth */
        DO:
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                sgMonthOut  = hChildNode:NODE-VALUE.
        END.

        IF hNode:NAME = "paramOut6" THEN /* gMonthDay */
        DO:
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                sgMonthDayOut  = hChildNode:NODE-VALUE.
        END.

        IF hNode:NAME = "paramOut7" THEN /* gYear */
        DO:
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                sgYearOut  = hChildNode:NODE-VALUE.
        END.
    
        IF hNode:NAME = "paramOut8" THEN /* gYearMonth */
        DO:
            Ret = hNode:get-child(hChildNode, 1).
            IF ret AND hChildNode:subtype = "TEXT" THEN 
                sgYearMonthOut  = hChildNode:NODE-VALUE.
        END.

    END.
  END.

  MESSAGE "Return Values = " SKIP
    lBoolOut SKIP 
    dtzDateOut SKIP  
    sdurationOut SKIP 
    sgDayOut SKIP 
    sgMonthOut SKIP
    sgMonthDayOut SKIP
    sgYearOut SKIP
    sgYearMonthOut VIEW-AS ALERT-BOX.
  END.
END.

DELETE PROCEDURE hPortType.
hWS:DISCONNECT(). 
DELETE OBJECT hWS.

PROCEDURE ErrorInfo:
  DEFINE OUTPUT PARAMETER errorfound AS LOGICAL INITIAL FALSE.
    
  DEFINE VARIABLE i AS INTEGER.
  DEFINE VARIABLE hSOAPFault AS HANDLE.
  DEFINE VARIABLE hSOAPFaultDetail AS HANDLE.
  DEFINE VARIABLE HeaderXML AS LONGCHAR VIEW-AS EDITOR SIZE 52 BY 5 LARGE.

  IF ERROR-STATUS:NUM-MESSAGES > 0 THEN 
  DO:
     errorfound = TRUE.
     DO i = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(i) VIEW-AS ALERT-BOX.
     END.
    
     IF VALID-HANDLE(ERROR-STATUS:ERROR-OBJECT-DETAIL) THEN
     DO:      
       hSOAPFault = ERROR-STATUS:ERROR-OBJECT-DETAIL.
       MESSAGE 
       "Fault Code: "   hSOAPFault:SOAP-FAULT-CODE        SKIP
       "Fault String: " hSOAPFault:SOAP-FAULT-STRING      SKIP
       "Fault Actor: "  hSOAPFault:SOAP-FAULT-ACTOR       SKIP
       "Error Type: "   hSOAPFault:TYPE VIEW-AS ALERT-BOX.

       IF VALID-HANDLE(hSOAPFault:SOAP-FAULT-DETAIL) THEN
        DO:
            hSOAPFaultDetail = hSOAPFault:SOAP-FAULT-DETAIL.
            MESSAGE  "Error Type: " hSOAPFaultDetail:TYPE VIEW-AS ALERT-BOX.
            HeaderXML = hSOAPFaultDetail:GET-SERIALIZED().
            DISPLAY HeaderXML LABEL "Serialized SOAP fault detail" WITH FRAME a.
        END.
     END. 
  END. 
END.


