/*
 * Class:    DynTT
 *
 */

import java.lang.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import DynTTSvc.*;
import DynTT.DynTTSvc.*;
import DynTT.DynTTSvc.holders.*;
import javax.xml.rpc.holders.*;
import org.apache.axis.message.MessageElement;
import org.apache.axis.AxisFault;

/**
 * This sample Web Services client application is built to use the client stubs
 * generated by the Apache Axis Beta-3 release WSDLtoJava utility.  The input
 * WSDL file is one generated by the WSA for the DynTTSrvc Web Service
 * Application that is based on the DynTT demo.
 * This is a Document/Literal, Session Free Web Service
 */
public class DynTT
{

    /*
     * Default Constructor
     */
    public DynTT()
    {
    }

    /*
     * Run method.
     */
    public void run(String[] args) throws Exception
    {

        // Generate the DynTT client's service object that manages the
        // connection with the WSA.
        //
        DynTTService    service = new DynTTServiceLocator();

        // Modify the target WSA's URL if doing special testing, like running
        // through a network proxy monitor to view the HTTP message traffic.
        //
        URL                 connectURL = new URL((args.length == 0) ? service.getDynTTObjAddress() : args[0]);

        System.out.println("connecting to URL: " + connectURL.toString());

        // Now generate the DynTT Web Service Application's
        // client object that will do all the marshalling and unmarshalling for
        // the SOAP and HTTP messages.
        //
        DynTTObjStub       dynTTObj = (DynTTObjStub)service.getDynTTObj(connectURL);

        try
        {
            // No need to make connection to the DynTT service, as it is a
            // Session Free Web Service

            // Build up a simple RowSet object.  The RowSet data is a Vector
            // of rows.  Each element of the row Vector is a Vector of columns.
            // The RowSet metaData is stored in the ColumnMetaData[] array

            RowSet rsIn = new RowSet();

            // Create the MetaData, consisting of a String, Integer and Date column
            ColumnMetaData[] schema = new ColumnMetaData[3];
            ColumnMetaData firstCol = new ColumnMetaData("Name", ColumnMetaData.CHAR_TYPE, 0);
            ColumnMetaData secondCol = new ColumnMetaData("Number", ColumnMetaData.INT_TYPE, 0);
            ColumnMetaData thirdCol = new ColumnMetaData("Birthday", ColumnMetaData.DATE_TYPE, 0);

            schema[0] = firstCol;
            schema[1] = secondCol;
            schema[2] = thirdCol;

            rsIn.setSchema(schema);

            // Create two data rows
            Vector row;

            row = new Vector();
            row.addElement(new String("Sally Jones"));
            row.addElement(new Integer(1));
            row.addElement(new GregorianCalendar(1998, 10, 23)); // month is zero based, this is November
            rsIn.addRow(row);

            row = new Vector();
            row.addElement(new String("George Smith"));
            row.addElement(new Integer(2));
            row.addElement(new GregorianCalendar(2001, 8, 15)); // month is zero based, this is September
            rsIn.addRow(row);

            // Convert the RowSet into a TableHandleParam Object,
            // the input parameter for DynTT.p

            // Get the RowSet as a DOM Element.  This element has two
            // children, a <schema> element and a <Data> element
            Element dataSetIn = rsIn.getDataSetAsDom();

            // Create a MessageElement containing the DataSet
            MessageElement dataSetMsgEl = new MessageElement(dataSetIn);

            // Place the DataSet MessageElement into the message element array
            // and create the TableHandleParam object
            MessageElement msgArrayIn[] = new MessageElement[1];
            msgArrayIn[0] = dataSetMsgEl;

            TableHandleParam tthIn = new TableHandleParam();
            tthIn.set_any(msgArrayIn);

           // Call DynTT.p
            StringHolder procResult = new StringHolder();
            TableHandleParamHolder ttHolder = new TableHandleParamHolder();

            dynTTObj.dynTT(tthIn, procResult, ttHolder);

            // Process the Output TABLE-HANDLE parameter, tthOut
            TableHandleParam tthOut = ttHolder.value;
            MessageElement [] msgArrayOut = tthOut.get_any();

            // msgArrayOut has one entry, the DataSet
            // The DataSet contains two elements, the <schema>
            // and the <Data>
            Element dataSetOut = msgArrayOut[0].getAsDOM();

            // Create a new RowSet Object based on the DataSet
            RowSet rsOut = new RowSet();

            rsOut.buildRowSet(dataSetOut);

            // print out the data for the dynamic temp-table to the console
            rsOut.printData();

        }
        catch (Exception e)
        {
            System.err.println("Caught Exception Type: " + e.getClass().getName());
            System.err.println("Exception getFaultCode: " + ((AxisFault)e).getFaultCode().toString());
            System.err.println("Exception getFaultString: " + ((AxisFault)e).getFaultString().toString());

	    // Get the list of child elements of the SOAP Fault <detail> element
    	    // There is only a single element allowed within the <detail>
    	    // element, which is the <FaultDetail> element.
    	    //
    	    Element[] faultDetails = ((AxisFault)e).getFaultDetails();
    	    if (faultDetails.length > 0)
    	    {
        	// Now get the contents of the <FaultDetail> element, which are the
        	// child elements <errorMessage> and <requestID>.  The first child
        	// of each element is a character node holding the String value.
        	//
        	Node      faultDetailElem = faultDetails[0].getFirstChild();
        	String    detailErrorMsg = faultDetailElem.getFirstChild().getNodeValue();
        	System.err.println("Exception errorMessage: " + detailErrorMsg);

        	Node      requestIDElem  = faultDetailElem.getNextSibling();
        	String    requestID      = requestIDElem.getFirstChild().getNodeValue();

        	System.err.println("Exception requestID   : " + requestID);
    	    }
    	    else
    	    {
        	// No <detail> element children can indicate that no more information
        	// is available.
        	//
        	System.err.println("Exception: no fault details");
    	    }

            System.err.println("Exception printStackTrace:...");
            e.printStackTrace();
        }

    }

    public static void main(String[] args)
    {
        DynTT  THIS = new DynTT();
        try
        {
            THIS.run(args);
        }
        catch(Exception e)
        {
            System.out.println("Error: " + e.toString());
        }
    }
}
