/*------------------------------------------------------------------------
    File        : DynTT.p
    Purpose     :

    Syntax      : DynTT (INPUT tthIN, 
                         OUTPUT tthOUT)

    Description : Sample procedure for demonstrating the use of Dynamic
                  TEMP-TABLES with Progress Web Services.  The sample
                  creates an OUTPUT TABLE-HANDLE, which is a copy
                  of the INPUT TABLE-HANDLE with one additional field.

    Author(s)   :
    Created     :
    Notes       :
  ----------------------------------------------------------------------*/


DEFINE INPUT PARAMETER TABLE-HANDLE tthIN.
DEFINE OUTPUT PARAMETER TABLE-HANDLE tthOUT.

DEFINE VARIABLE bhIN AS HANDLE.
DEFINE VARIABLE bhOUT AS HANDLE.
DEFINE VARIABLE qh AS HANDLE.
DEFINE VARIABLE i AS INTEGER.


bhIN = tthIN:DEFAULT-BUFFER-HANDLE.

/* Create the OUTPUT TABLE-HANDLE.  Copy the definition of the 
** INPUT TABLE-HANDLE, and add a character field */
CREATE TEMP-TABLE tthOUT.
tthOUT:ADD-FIELDS-FROM(bhIN).

tthOUT:ADD-NEW-FIELD("newColumn", "CHARACTER").

tthOUT:TEMP-TABLE-PREPARE("outTable").

bhOUT = tthOUT:DEFAULT-BUFFER-HANDLE.


/* Loop through the INPUT's records, copying them to the OUTPUT */
CREATE QUERY qh.

qh:SET-BUFFERS(bhIN).

qh:QUERY-PREPARE("FOR EACH " + bhIN:NAME).
qh:QUERY-OPEN.

i = 1.

REPEAT:
    qh:GET-NEXT.
    IF qh:QUERY-OFF-END THEN LEAVE.

    bhOUT:BUFFER-CREATE.
    bhOUT:BUFFER-COPY(bhIN).

    /* Assign data to the two new fields */
    bhOUT:BUFFER-FIELD("newColumn"):BUFFER-VALUE = "newChar" + STRING(i).

    i = i + 1.
END.


bhIN:BUFFER-RELEASE.
bhOUT:BUFFER-RELEASE.

DELETE OBJECT tthOUT.
DELETE OBJECT qh.
