/*
 * SOAPHeaderWSTK-PO.p
 *
 * Calls the OrderInfo Web Service Sample which is a session-managed
 * Web Service containing a ProcObject.
 *
 * This example calls functions inside of the ProcObject. 
 *
 * The web service connection, port setting, and invocation code
 * are all based on information from the OpenEdges WSDL Analyser
 * (bprowsdldoc).
 *
 */

/* Define local variables */
DEFINE VARIABLE hWebService   AS HANDLE NO-UNDO.
DEFINE VARIABLE hWebServicePO   AS HANDLE NO-UNDO.
DEFINE VARIABLE hOrderInfoObj AS HANDLE NO-UNDO.
DEFINE VARIABLE hCustomerOrderObj AS HANDLE NO-UNDO.
DEFINE VARIABLE appObj_header   AS HANDLE NO-UNDO.
DEFINE VARIABLE custordPO_header  AS HANDLE NO-UNDO.
DEFINE VARIABLE iCustNum AS INTEGER INITIAL 46 NO-UNDO.
DEFINE VARIABLE cCustName AS CHARACTER NO-UNDO.
DEFINE VARIABLE cUserId   AS CHARACTER NO-UNDO.
DEFINE VARIABLE cPassword  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cAppSrvrInfo  AS CHARACTER NO-UNDO.
DEFINE VARIABLE custNum AS INTEGER INITIAL 1 NO-UNDO.
DEFINE VARIABLE cresult AS CHARACTER NO-UNDO.
DEFINE VARIABLE dThreshold AS DECIMAL INITIAL 0 NO-UNDO.
DEFINE VARIABLE iresult AS INTEGER NO-UNDO.
DEFINE VARIABLE dresult AS DECIMAL NO-UNDO.
DEFINE VARIABLE idx AS INTEGER NO-UNDO.


FUNCTION CreatePO_CustomerOrder RETURNS CHARACTER (INPUT INTEGER) IN hOrderInfoObj.
FUNCTION GetTotalOrdersByNumber RETURNS INTEGER (INPUT DECIMAL) IN hCustomerOrderObj.
FUNCTION GetTotalOrdersByDollar RETURNS DECIMAL (INPUT DECIMAL) IN hCustomerOrderObj.

/* Create Web Service for the AppObject */
CREATE SERVER hWebService.

/* Connect to the WS for the AppObject*/
hWebService:CONNECT("-WSDL  http://localhost:8080/wsa/wsa1/wsdl?targetURI=urn:OrderInfoSvc
                     -Service OrderInfoService
                     -Port OrderInfoObj").
IF NOT hWebService:CONNECTED() THEN DO: 
    MESSAGE "Connect to OrderInfo failed!" VIEW-AS ALERT-BOX.
    RUN cleanup.
    LEAVE.
END.

/* Set the handle to the port type */
RUN OrderInfoObj SET hOrderInfoObj ON hWebService.
IF NOT VALID-HANDLE(hOrderInfoObj) THEN DO: 
    MESSAGE "Unable to get handle to OrderInfo PortType!" VIEW-AS ALERT-BOX.
    RUN cleanup.
    LEAVE.
END.

/* Associate the request & response callbacks with the port type */
hOrderInfoObj:SET-CALLBACK-PROCEDURE("REQUEST-HEADER", "RequestHandler"). 
hOrderInfoObj:SET-CALLBACK-PROCEDURE("RESPONSE-HEADER", "ResponseHandler").

/* First call to send an "empty" header for connection information
   for the next call */
RUN Connect_OrderInfo IN hOrderInfoObj(INPUT cUserId, INPUT cPassword, 
                                       INPUT cAppSrvrInfo) NO-ERROR.

IF ERROR-STATUS:ERROR THEN DO:
    MESSAGE "Connect to the OrderInfo service failed!" VIEW-AS ALERT-BOX.
    DO idx = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(idx) VIEW-AS ALERT-BOX.
    END.
    RUN cleanup.
    LEAVE.
END.

/* Fetch the customer name based on the value of iCustNum,
   using SOAP header infomation from the previous call */
RUN FindCustomerByNum IN hOrderInfoObj (INPUT iCustNum, OUTPUT cresult, 
                                        OUTPUT cCustName) NO-ERROR.

IF ERROR-STATUS:ERROR THEN DO:
    MESSAGE "Run of FindCustomerByNum failed!" VIEW-AS ALERT-BOX.
    DO idx = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(idx) VIEW-AS ALERT-BOX.
    END.
    RUN cleanup.
    LEAVE.
END.

MESSAGE "WS response from FindCustomerByNum is: " SKIP cCustName VIEW-AS ALERT-BOX.

/* Call CreatePO_CustomerOrder */
cresult = CreatePO_CustomerOrder(custNum) NO-ERROR.
IF ERROR-STATUS:NUM-MESSAGES > 0 THEN DO:
    MESSAGE "CreatePO_CustomerOrder failed!" VIEW-AS ALERT-BOX.
    DO idx = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(idx) VIEW-AS ALERT-BOX.
    END.
    RUN cleanup.
    LEAVE.
END.

/* Create Web Service for the ProcObject CustomerOrder */
CREATE SERVER hWebServicePO.

/* Connect to the WS for the ProcObject CustomerOrder */
hWebServicePO:CONNECT("-WSDL  http://localhost:8080/wsa/wsa1/wsdl?targetURI=urn:OrderInfoSvc
                     -Service OrderInfoService
                     -Port CustomerOrderObj ").

IF NOT hWebServicePO:CONNECTED() THEN DO: 
    MESSAGE "Connect to CustomerOrder failed!" VIEW-AS ALERT-BOX.
    RUN cleanup.
    LEAVE.
END.


/* Set the handle to the port type */
RUN CustomerOrderObj SET hCustomerOrderObj ON hWebServicePO.
IF NOT VALID-HANDLE(hCustomerOrderObj) THEN DO: 
    MESSAGE "Unable to get handle to CustomerOrder PortType!" VIEW-AS ALERT-BOX.
    RUN cleanup.
    LEAVE.
END.

/* Associate the request & response callbacks with the port type */
hCustomerOrderObj:SET-CALLBACK-PROCEDURE("REQUEST-HEADER",  "RequestHandler"). 
hCustomerOrderObj:SET-CALLBACK-PROCEDURE("RESPONSE-HEADER", "ResponseHandler").

/*run operations in the persistent procedure */
iresult = GetTotalOrdersByNumber(dThreshold) NO-ERROR.

IF ERROR-STATUS:NUM-MESSAGES > 0 THEN DO:
    MESSAGE "GetTotalOrdersByNumber failed!" VIEW-AS ALERT-BOX.
    DO idx = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(idx) VIEW-AS ALERT-BOX.
    END.
    RUN cleanup.
    LEAVE.
END.

MESSAGE "The total Orders by number for customer " + STRING(custNum) + ": " SKIP iresult VIEW-AS ALERT-BOX.

dresult = GetTotalOrdersByDollar(dThreshold) NO-ERROR.

IF ERROR-STATUS:NUM-MESSAGES > 0 THEN DO:
    MESSAGE "GetTotalOrdersByDollar failed!" VIEW-AS ALERT-BOX.
    DO idx = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(idx) VIEW-AS ALERT-BOX.
    END.
    RUN cleanup.
    LEAVE.
END.

MESSAGE "The total Orders by dollar for customer " + STRING(custNum) + ": " SKIP dresult VIEW-AS ALERT-BOX.

RUN cleanup.

/**************** Internal Procedures ****************/
PROCEDURE RequestHandler:
    DEFINE OUTPUT PARAMETER hSOAPHeader    AS HANDLE.
    DEFINE INPUT  PARAMETER cOperationNS   AS CHARACTER.
    DEFINE INPUT  PARAMETER cOperationLocalName AS CHARACTER.
    DEFINE OUTPUT PARAMETER lDeleteOnDone  AS LOGICAL.
    
    CASE cOperationLocalName :
        WHEN "FindCustomerByNum" THEN hSOAPHeader = appObj_header.
        WHEN "CreatePO_CustomerOrder" THEN hSOAPHeader = appObj_header.
        WHEN "GetTotalOrdersByNumber" THEN hSOAPHeader = custordPO_header.
        WHEN "GetTotalOrdersByDollar" THEN hSOAPHeader = custordPO_header.
        WHEN "Release_CustomerOrder" THEN 
            DO:
                hSOAPHeader = custordPO_header.
                lDeleteOnDone = YES.
            END.
        WHEN "Release_OrderInfo" THEN 
            DO:
                hSOAPHeader = appObj_header.
                lDeleteOnDone = YES.
            END.
    END CASE.
END PROCEDURE.

PROCEDURE ResponseHandler:
    DEFINE INPUT PARAMETER hSOAPHeader    AS HANDLE.
    DEFINE INPUT PARAMETER cOperationNS   AS CHARACTER.
    DEFINE INPUT PARAMETER cOperationLocalName AS CHARACTER.

    CASE cOperationLocalName :
        WHEN "Connect_OrderInfo" THEN appObj_header = hSOAPHeader.
        WHEN "CreatePO_CustomerOrder" THEN custordPO_header = hSOAPHeader.
        OTHERWISE 
            DELETE OBJECT hSOAPHeader. /* Delete all unused soap headers */
    END CASE.
END PROCEDURE.

PROCEDURE cleanup:
    /* release the objects */
    IF VALID-HANDLE(hCustomerOrderObj) THEN
        RUN Release_CustomerOrder IN hCustomerOrderObj.

    IF VALID-HANDLE(hOrderInfoObj) THEN
        RUN Release_OrderInfo IN hOrderInfoObj.
    
    /* free up memory associated with the Server and Procedure objects */
    DELETE OBJECT hCustomerOrderObj NO-ERROR.
    hWebServicePO:DISCONNECT() NO-ERROR.
    DELETE OBJECT hWebServicePO NO-ERROR.
    
    DELETE OBJECT hOrderInfoObj NO-ERROR.
    hWebService:DISCONNECT() NO-ERROR.
    DELETE OBJECT hWebService NO-ERROR.

END PROCEDURE.
