/*
 * SOAPHeaderWSTK.p
 *
 * Call the FindCustomerByNum sample method that ships with the 
 * OrderInfo OpenEdge Web services sample.
 *
 * The web service connection, port setting, and invocation code
 * are all based on information from the OpenEdge WSDL Analyzer
 * (bprowsdldoc).
 *
 */

/* Define local variables */
DEF VAR hWebService   AS HANDLE NO-UNDO.
DEF VAR hOrderInfoObj AS HANDLE NO-UNDO.
DEF VAR g_header      AS HANDLE NO-UNDO.
DEF VAR iCustNum      AS INTEGER INIT 46 NO-UNDO.
DEF VAR cCustName     AS CHARACTER NO-UNDO.
DEF VAR cUserId       AS CHARACTER NO-UNDO.
DEF VAR cPassword     AS CHARACTE NO-UNDO.
DEF VAR cAppSrvrInfo  AS CHARACTER NO-UNDO.
DEF VAR idx           AS INTEGER NO-UNDO.
DEF VAR cresult       AS CHARACTER NO-UNDO.

/* Create stuff */
CREATE SERVER hWebService.

/* Connect to the WS */
hWebService:CONNECT("-WSDL  http://localhost:8080/wsa/wsa1/wsdl?TargetURI=urn:OrderInfoSvc
                     -Service OrderInfoService
                     -Port OrderInfoObj").
IF NOT hWebService:CONNECTED() THEN LEAVE.

/* Get the method, set the port type */
RUN OrderInfoObj SET hOrderInfoObj ON hWebService.
IF NOT VALID-HANDLE(hOrderInfoObj) THEN LEAVE.

/* Associate the request & response callbacks with the port type */
hOrderInfoObj:SET-CALLBACK-PROCEDURE("REQUEST-HEADER", "ReqHandler"). 
hOrderInfoObj:SET-CALLBACK-PROCEDURE("RESPONSE-HEADER", "ResHandler").

/* First call to send an "empty" header for connection information
   for the next call */
RUN Connect_OrderInfo IN hOrderInfoObj(INPUT cUserId, INPUT cPassword, 
                                       INPUT cAppSrvrInfo) NO-ERROR.
IF ERROR-STATUS:ERROR THEN DO:
    MESSAGE "Connect to the OrderInfo service failed" VIEW-AS ALERT-BOX.
    LEAVE.
END.

/* Fetch the customer name based on the value of iCustNum,
   using SOAP header infomation from the previous call */
RUN FindCustomerByNum IN hOrderInfoObj (INPUT iCustNum, OUTPUT cresult, OUTPUT cCustName) NO-ERROR.
IF ERROR-STATUS:ERROR THEN DO:
    MESSAGE "Run of FindCustomerByNum failed!" VIEW-AS ALERT-BOX.
    DO idx = 1 TO ERROR-STATUS:NUM-MESSAGES:
        MESSAGE ERROR-STATUS:GET-MESSAGE(idx) VIEW-AS ALERT-BOX.
    END.
    RUN cleanup.
    LEAVE.
END.

MESSAGE "WS response is: " SKIP
        cCustName VIEW-AS ALERT-BOX.

RUN cleanup.

/**************** Internal Procedures ****************/
PROCEDURE ReqHandler:
    DEF OUTPUT PARAMETER hHeader       AS HANDLE.
    DEF INPUT  PARAMETER cNamespace    AS CHAR.
    DEF INPUT  PARAMETER cLocalNS      AS CHAR.
    DEF OUTPUT PARAMETER lDeleteOnDone AS LOGICAL.

    hHeader = g_header.
    IF cLocalNS = "Release_OrderInfo" THEN
        lDeleteOnDone = YES.
END PROCEDURE.

PROCEDURE ResHandler:
    DEF INPUT PARAMETER hHeader    AS HANDLE.
    DEF INPUT PARAMETER cNamespace AS CHAR.
    DEF INPUT PARAMETER cLocalNS   AS CHAR.

    IF cLocalNS = "Connect_OrderInfo" THEN
        g_header = hHeader.
    ELSE
        /* Delete all unused soap headers */
        DELETE OBJECT hHeader. 
END PROCEDURE.

PROCEDURE cleanup:
    RUN Release_OrderInfo IN hOrderInfoObj.
    DELETE OBJECT hOrderInfoObj.
    hWebService:DISCONNECT().
    DELETE OBJECT hWebService.
END PROCEDURE.
