/*
 * Class:    OrderInfo
 *
 * Date:     10/30/2008
 * Time:     2:04:36 PM
 */


import java.net.*;

import java.math.BigDecimal;
import OrderInfoSvc.*;
import CustomerOrder.OrderInfoSvc.*;
import CustomerOrder.OrderInfoSvc.holders.*;
import javax.xml.rpc.holders.*;
import org.apache.axis.AxisFault;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * This sample Web Services client application is built to use the client stubs
 * generated by the Apache Axis 1.4 release WSDLtoJava utility.  The input
 * WSDL file is one generated by the WSA for the OrderService Web Service 
 * Application that is based on the OrderInfo demo.
 */
public class OrderInfo
{   
    /*
     * Default Constructor
     */
    public OrderInfo()
    {
    }

    /*
     * Run method.
     */
    public void run(String[] args) throws Exception
    {
        
        // Generate the OrderInfo client's service object that manages the
        // connection with the WSA.
        //
        OrderInfoService    service = new OrderInfoServiceLocator();
        
        // Modify the target WSA's URL if doing special testing, like running
        // through a network proxy monitor to view the HTTP message traffic.
        //
        URL connectURL = new URL((args.length == 0) ? service.getOrderInfoObjAddress() : args[0]);
        
        System.out.println("connecting to URL: " + connectURL.toString());

        // Do not turn on debug messages from the handler.
        //
        boolean             debugHandler = false;
        
        // Setup a couple of reminders that we need to call a release method
        // for each Proxy Object we create.
        //
        boolean             releaseOrderInfo = false;
        boolean             releaseCustomerOrder = false;

        // Define the names the Object ID handler will scan for.  Because some
        // operations send out an ID for one object and get back an object for
        // a newly created object, the ID handler needs to know about all of
        // them.
        //
        String[]            orderInfoObjectNames = new String[] {"OrderInfo",
                                                                 "CustomerOrder" };
                                                                 
        // Insert the handlers used to parse the Object Identifiers issued by
        // the WSA from the incomming SOAP response messages and inserting 
        // those same Object Identifiers into the outgoing SOAP request messages.
        //
        HandlerControlBlock custOrderControlBlock = null;
        HandlerControlBlock orderInfoControlBlock = null;
        orderInfoControlBlock = 
              PscObjectIDHandler.registerObjectIDHandler(service, 
                                                         orderInfoObjectNames, 
                                                         "",
                                                         "OrderInfoPort", 
                                                         debugHandler);
        custOrderControlBlock = 
	      PscObjectIDHandler.registerObjectIDHandler(service, 
                                                         orderInfoObjectNames, 
                                                         "",
                                                         "CustomerOrderPort", 
                                                         debugHandler);

        // Now generate the OrderInfo & CustomerOrder Web Service Application's 
        // client objects that will do all the marshalling and unmarshalling for
        // the SOAP and HTTP messages.
        //
        OrderInfoObjStub       orderInfo = (OrderInfoObjStub)service.getOrderInfoObj(connectURL);
        CustomerOrderObjStub   custOrder = (CustomerOrderObjStub)service.getCustomerOrderObj(connectURL);

        // Set the port type name manually for an Axis client, as the stubs
        // generated by the wsdl document to not do it (yet).
        //
        orderInfo.setPortName("OrderInfoPort");
        custOrder.setPortName("CustomerOrderPort");
                                                 
        try
        {
            // Make the logical connection to the server OrderInfo object...
            //
            orderInfo.connect_OrderInfo("", "", "");

            // Get the OrderInfo object's ID for later use.
            //
            if (null == orderInfoControlBlock.getObjectNameID("OrderInfo"))
            {
                throw new Exception("No header returned from a connect operation to OrderInfo.");   
            }
        
            // System.out.println("OrderInfo object connection returned ID: " + 
            //                    orderInfoControlBlock.getObjectNameID("OrderInfo"));

            // Remember, we have to release the Proxy object.
            releaseOrderInfo = true;

        
            // Find a customer to see that we can obtain information.
            // The orderinfo Object Identifier is now set in the handler,
            // so that it can just be reused.
            StringHolder    custNameHolder = new StringHolder();
            StringHolder    resultHolder = new StringHolder();
            int             custNum = 3;
            orderInfo.findCustomerByNum(custNum, resultHolder, custNameHolder);
            System.out.println("Customer #3's name is " + custNameHolder.value);

            // Now get the inner persistent procedure Object (CustomerOrder)
            // Note, the WSA will issue a new Object Identifier for this new
            // object instance.  
            //
            // Creating the persistent procedure object will result in overwriting
            // the orderInfo object id that was stored with the new one for the
            // persistent procedure object.  Setup to grab the new one as it
            // comes in.
            //
            String result = orderInfo.createPO_CustomerOrder(custNum);

            // Save the new persistent object id...
            // The object id passed in subsequent requests will point to the
            // persistent object.  We can leave it as is until we have to call
            // a method on another object.
            //
            // Now test the object id for the customer order object...
            if (null == orderInfoControlBlock.getObjectNameID("CustomerOrder"))
            {
                throw new Exception("No header returned from a create operation for CustomerOrder.");   
            }
        
            // System.out.println("CustomerOrder object connection returned ID: " +
            //             orderInfoControlBlock.getObjectNameID("CustomerOrder"));

            // Set the handler to send the CustomerOrder's object ID on all 
            // subsequent Web Service Requests (until we tell it otherwise).
            //
//            handlerControlBlock.setRequestObjectIDName("CustomerOrder");
            custOrderControlBlock.importObjectID("CustomerOrder", 
                                                 orderInfoControlBlock.exportObjectID("CustomerOrder"));
            orderInfoControlBlock.releaseObjectNameID("CustomerOrder");
            
            // Remember, we have to release the Proxy object.
            releaseCustomerOrder = true;
            
            
            // make a call to the persistent procedure to verify it's 
            // creation and operation.
            //
            try
            {
                int orderCount = custOrder.getTotalOrdersByNumber(new BigDecimal(5.0));
                System.out.println("The total Customer Orders are: " + orderCount);

                // Now let's verify that temp-tables can be passed back.
                // Create the holder for the temp-table OrderDetails
		        OrderDetailsParamHolder orderDetailsHolder =
		                                              new OrderDetailsParamHolder(null);

                // Let's now ask the WebService for the order details of the 
                // current customer
	            custOrder.getOrderDetails(resultHolder, orderDetailsHolder);

                // .. And print them out

	            OrderDetailsRow[] rows = (OrderDetailsRow[]) orderDetailsHolder.value;

                // Print out some labels
                System.out.println();
                System.out.println("Customer's Order Details listed below:");
                System.out.println();
		        System.out.println("Order#  " +
				   "Order Date                     " +
				   "Rep   " +
				   "Order Status        " +
				   "Total Dollars");
                System.out.println();

                // Now print out all the rows
		        for (int j = 0; j < rows.length; j++)
		        {
		        	OrderDetailsRow thisRow = rows[j];
                    String orderNumbStr, ordNumFiller, statusFiller;

                    orderNumbStr = (thisRow.getOrderNum()).toString();

		            if (orderNumbStr.length() == 1)
                        ordNumFiller = "       ";
		            else if (orderNumbStr.length() == 2)
                        ordNumFiller = "      ";
		            else
                        ordNumFiller = "     ";

		            if (thisRow.getOrderStatus().equals("Partially Shipped"))
                        statusFiller = "   ";
                    else
                        statusFiller = "             ";


		            System.out.println(thisRow.getOrderNum() + ordNumFiller +
				       thisRow.getOrderDate() + "   " +
				       thisRow.getSalesRep() + "   " +
				       thisRow.getOrderStatus() + statusFiller +
				       thisRow.getTotalDollars());
		        }
		        
                System.out.println();

            }
            catch (NullPointerException con)
            {
                // It is possible in Progress server applications to return an
                // "Unknown" from functions that claim to return an atomic
                // data type.  This shows up as a null value, which can cause
                // an exception.  This handler will compensate for this fact.
                //
                System.err.println("The total Customer Orders are: 0 (unknown)");
            }
            
            // Now release the persistent procedure object CustomerOrder
            //
            custOrder.release_CustomerOrder();

            releaseCustomerOrder = false;
            
            // Remove the object id that we just released so that it can't be
            // used again.
            //
            custOrderControlBlock.releaseObjectNameID("CustomerOrder");
        
            // The last step is to release the OrderInfo object's logical 
            // connection, we're done.
            // We now need to place the order info object's id back into the
            // id handler since we're now calling methods on orderinfo object
            // again.
            //
            // handlerControlBlock.setRequestObjectIDName("OrderInfo");
            
            orderInfo.release_OrderInfo();
        
            releaseOrderInfo = false;
            
            // Remove the object id that we just released so that it can't be
            // used again.
            //
            orderInfoControlBlock.releaseObjectNameID("OrderInfo");
        
            // Ah, we're done now...
        }
        catch (Exception e)
        {
            System.err.println("Caught Exception Type: " + e.getClass().getName());
            System.err.println("Exception getFaultCode: " + ((AxisFault)e).getFaultCode().toString());
            System.err.println("Exception getFaultString: " + ((AxisFault)e).getFaultString().toString());


	        // Get the list of child elements of the SOAP Fault <detail> element.
    	    // There is only a single element allowed within the <detail> 
    	    // element, which is the <FaultDetail> element.
    	    //
    	    Element[] faultDetails = ((AxisFault)e).getFaultDetails();
    	    if (faultDetails.length > 0)
    	    {
        	    // Now get the contents of the <FaultDetail> element, which are the
        	    // child elements <errorMessage> and <requestID>.  The first child
        	    // of each element is a character node holding the String value.
        	    //
        	    Node      faultDetailElem = faultDetails[0].getFirstChild();
        	    String    detailErrorMsg = faultDetailElem.getFirstChild().getNodeValue(); 
        	    System.err.println("Exception errorMessage: " + detailErrorMsg);

        	    Node      requestIDElem  = faultDetailElem.getNextSibling();
        	    String    requestID      = requestIDElem.getFirstChild().getNodeValue();
        
        	    System.err.println("Exception requestID   : " + requestID);
    	    }
    	    else
    	    {
        	    // No <detail> element children can indicate that no more information
        	    // is available.
        	    //
        	    System.err.println("Exception: no fault details");
    	    }

            System.err.println("Cleaning up connections...");
            
            // Attempt to do the releases to cleanly close any open logical
            // connections.  Always release from the inner objects 
            // (CustomerOrder)outward to the primary application 
            // object (OrderInfo).
            //
            if (releaseCustomerOrder == true)
            {
                try
                {
                    custOrder.release_CustomerOrder();
                }
                catch (Exception e1)
                {
                    System.err.println("Unable to release the CustomerOrder " +
                                       "connection cleaning up after an error.");
                    System.err.println(e.toString());
                }
            }    
            
            if (releaseOrderInfo == true)
            {
                try
                {
                    orderInfo.release_OrderInfo();
                }
                catch (Exception e1)
                {
                    System.err.println("Unable to release the CustomerOrder " +
                                       "connection cleaning up after an error.");
                    System.err.println(e.toString());
                }
            }    
        }
    }

    public static void main(String[] args)
    {
        OrderInfo  THIS = new OrderInfo();
        try
        {
            THIS.run(args);
        }
        catch(Exception e)
        {
            System.out.println("Error: " + e.toString());
        }
    }
}
