&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v9r12
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Procedure 
/*------------------------------------------------------------------------
    File        : CustomerOrder.p 
    Purpose     :

    Syntax      :

    Description : Persistent Procedure that manages the details about a 
                  customer's orders.

    Author(s)   :
    Created     :
    Notes       :
  ----------------------------------------------------------------------*/
/*          This .W file was created with the Progress AppBuilder.      */
/*----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
DEFINE INPUT PARAMETER custNum AS INTEGER NO-UNDO.

DEFINE VARIABLE thisCust AS INTEGER NO-UNDO.

DEFINE TEMP-TABLE OrderDetails
    FIELD OrderNum LIKE Order.OrderNum
    FIELD SalesRep LIKE Order.SalesRep
    FIELD OrderDate LIKE Order.OrderDate
    FIELD ShipDate LIKE Order.ShipDate
    FIELD TotalDollars AS DECIMAL
    FIELD OrderStatus LIKE Order.OrderStatus.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no



/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME


/* ************************  Function Prototypes ********************** */

&IF DEFINED(EXCLUDE-GetTotalDollars) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD GetTotalDollars Procedure 
FUNCTION GetTotalDollars RETURNS DECIMAL PRIVATE
  (OrderNum AS INTEGER)  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-GetTotalOrdersByDollar) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD GetTotalOrdersByDollar Procedure 
FUNCTION GetTotalOrdersByDollar RETURNS DECIMAL
  (Threshold AS DECIMAL)  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-GetTotalOrdersByNumber) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD GetTotalOrdersByNumber Procedure 
FUNCTION GetTotalOrdersByNumber RETURNS INTEGER
  (Threshold AS DECIMAL)  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF


/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Procedure
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: CODE-ONLY COMPILE
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 15
         WIDTH              = 60.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Procedure 


/* ***************************  Main Block  *************************** */

thisCust = custNum.
FIND FIRST Customer WHERE Customer.CustNum = thisCust NO-ERROR.
IF AVAILABLE Customer THEN
    RETURN Customer.NAME.
ELSE
    RETURN ?.


/*FUNCTION GetOrderDetails RETURNS LOGICAL (OUTPUT TABLE FOR OrderDetails).*/

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&IF DEFINED(EXCLUDE-GetOrderDetails) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE GetOrderDetails Procedure 
PROCEDURE GetOrderDetails :
/*------------------------------------------------------------------------------
  Purpose: Returns an output temp-table that contains the details of all the orders
           a particular customer has made.    
  Parameters:  OrderDetails
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE OUTPUT PARAMETER TABLE FOR OrderDetails.

    EMPTY TEMP-TABLE OrderDetails.

    FOR EACH Order WHERE Order.CustNum = thisCust:
        CREATE OrderDetails.
        ASSIGN 
            OrderDetails.OrderNum = Order.OrderNum
            OrderDetails.SalesRep = Order.SalesRep
            OrderDetails.OrderDate = Order.OrderDate
            OrderDetails.ShipDate = Order.ShipDate
            OrderDetails.OrderStatus = Order.OrderStatus.

        OrderDetails.TotalDollars = GetTotalDollars(Order.OrderNum).
    END.
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

/* ************************  Function Implementations ***************** */

&IF DEFINED(EXCLUDE-GetTotalDollars) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION GetTotalDollars Procedure 
FUNCTION GetTotalDollars RETURNS DECIMAL PRIVATE
  (OrderNum AS INTEGER) :
/*------------------------------------------------------------------------------
  Purpose: Returns the total dollars of a particular order 
    Notes:  
------------------------------------------------------------------------------*/
    DEFINE VARIABLE TotalDollars AS DECIMAL NO-UNDO INITIAL 0.0.

    FOR EACH OrderLine WHERE OrderLine.OrderNum = OrderNum:
        TotalDollars = TotalDollars + OrderLine.ExtendedPrice.
    END.
    RETURN TotalDollars.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-GetTotalOrdersByDollar) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION GetTotalOrdersByDollar Procedure 
FUNCTION GetTotalOrdersByDollar RETURNS DECIMAL
  (Threshold AS DECIMAL) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
    DEFINE VARIABLE DollarCount AS DECIMAL INITIAL 0 NO-UNDO.
    DEFINE VARIABLE TotalDollars AS DECIMAL INITIAL 0 NO-UNDO.

    FOR EACH Order WHERE Order.CustNum = thisCust:
        TotalDollars = GetTotalDollars(Order.OrderNum).
        IF TotalDollars > Threshold THEN DollarCount = DollarCount + TotalDollars.
    END.

    RETURN DollarCount.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-GetTotalOrdersByNumber) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION GetTotalOrdersByNumber Procedure 
FUNCTION GetTotalOrdersByNumber RETURNS INTEGER
  (Threshold AS DECIMAL) :
/*------------------------------------------------------------------------------
  Purpose: Returns the total number of orders for this customer that exceed a 
           specified threshold. 
    Notes:  
------------------------------------------------------------------------------*/
    DEFINE VARIABLE OrderCount AS INTEGER INITIAL 0 NO-UNDO.
    DEFINE VARIABLE TotalDollars AS DECIMAL INITIAL 0 NO-UNDO.

    FOR EACH Order WHERE Order.CustNum = thisCust:
        TotalDollars = GetTotalDollars(Order.Ordernum).
        IF TotalDollars > Threshold THEN OrderCount = OrderCount + 1.
    END.

    RETURN OrderCount.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

