/*
 * SOAPHeader1.p
 *
 * Calls a ficticious Web service, first to request access, which 
 * gets back a SOAP response header containing an AccessID,
 * and sends the response header back as part of a new request 
 * using the required access credential that allows the Web service 
 * to respond appropriately to the follow-up request. 
 * 
 * The Web service has only one service and port available.
 *
 */

/* Define local variables */
DEFINE VARIABLE hWebSrvc    AS HANDLE.
DEFINE VARIABLE hPortType   AS HANDLE.
DEFINE VARIABLE cResponse   AS CHAR FORMAT "x(72)".
DEFINE VARIABLE g_header    AS HANDLE.

/* Create the Web service server object */
CREATE SERVER hWebSrvc.

/* Connect to the Web service */
hWebSrvc:CONNECT("-WSDL http://ServiceHost/SOAPHeader/HeaderExample.asmx?wsdl").

/* Get the method, set the port type */
RUN HeadersSoap SET hPortType ON hWebSrvc.

/* Associate the req. & resp. callbacks with the port type */
hPortType:SET-CALLBACK-PROCEDURE("REQUEST-HEADER", "ReqHandler").
hPortType:SET-CALLBACK-PROCEDURE("RESPONSE-HEADER", "RespHandler").

/* Invoke the Web service with no header and display the results */
RUN OpenAccess IN hPortType (OUTPUT cResponse).
DISPLAY cResponse LABEL "WS response" WITH FRAME aaa.

/* Go again with the AccessID set from previous response header */
cResponse = "".
RUN HelloWorld IN hPortType (OUTPUT cResponse).
DISPLAY cResponse LABEL "WS response" WITH FRAME bbb.

DELETE OBJECT g_header.
DELETE OBJECT hPortType.
hWebSrvc:DISCONNECT().
DELETE OBJECT hWebSrvc.

/**************** Internal Procedures ****************/
PROCEDURE ReqHandler:
    DEFINE OUTPUT PARAMETER hHeader    AS HANDLE.
    DEFINE INPUT  PARAMETER cNamespace AS CHAR.
    DEFINE INPUT  PARAMETER cLocalNS   AS CHAR.
    DEFINE OUTPUT PARAMETER lDeleteOnDone AS LOGICAL.

        /* The IF test determines if this is the first request.
           If it is, then g_header is not set and hHeader is set
           to ? to ensure that no header is sent.
           g_header gets set when the response header is
           returned, so a subsequent pass through this code takes
           the previous response header and sends it as the
           current request header. */ 
    IF NOT VALID-HANDLE (g_header) THEN DO: /* first request */
        hHeader = ?.
        lDeleteOnDone = TRUE.
    END.
    ELSE DO: /* all subsequent requests */
        hHeader = g_header.
        lDeleteOnDone = FALSE.
    END.
END PROCEDURE.

PROCEDURE RespHandler:
    DEFINE INPUT PARAMETER hHeader    AS HANDLE.
    DEFINE INPUT PARAMETER cNamespace AS CHAR.
    DEFINE INPUT PARAMETER cLocalNS   AS CHAR.
        
        /* If the g_header global variable 
           is valid coming in, it has already been set 
           in a previous response, therefore, delete 
           the unnecessary response header object.
           Otherwise, set g-header to the response 
           header object to pass back to the request 
           header handler on subsequent requests. */
    IF VALID-HANDLE(g_header) THEN
        DELETE OBJECT hHeader.
    ELSE DO:
        g_header = hHeader.
    END.    
END PROCEDURE.
