/*
 * SOAPHeader3.p
 *
 * Calls a ficticious web service, passes it a username
 * & password via a SOAP message request header, and gets
 * back a string.
 * 
 * The Web service has only one service and port available.
 *
 */

/* Define local variables */
DEFINE VARIABLE hWebSrvc      AS HANDLE.
DEFINE VARIABLE hPortType     AS HANDLE.
DEFINE VARIABLE cUsername     AS CHARACTER INIT "Scott".
DEFINE VARIABLE cPassword     AS CHARACTER INIT "Administrator".
DEFINE VARIABLE cResponse     AS CHARACTER FORMAT "x(72)".
DEFINE VARIABLE hXdoc         AS HANDLE.
DEFINE VARIABLE hXnoderef1    AS HANDLE.
DEFINE VARIABLE hXnoderef2    AS HANDLE.
DEFINE VARIABLE hXtext        AS HANDLE.
DEFINE VARIABLE lDeleteOnDone AS LOGICAL.
DEFINE VARIABLE g_header      AS HANDLE.

/* Build global SOAP request header */
RUN BuildRequestHeader (OUTPUT g_header).

/* Create the Web service server object */
CREATE SERVER  hWebSrvc.

/* Connect to the WS */
hWebSrvc:CONNECT("-WSDL http://ServiceHost/SOAPHeader/HeaderExample.asmx?wsdl").

/* Get the method, set the port type */
RUN HeaderExampleSoap SET hPortType ON hWebSrvc.

/* Associate the request callback with the port type */
hPortType:SET-CALLBACK-PROCEDURE("REQUEST-HEADER", "ReqHandler").

/* Invoke the web service and display the results */
RUN HellowMyWorld IN hPortType (OUTPUT cResponse).
DISPLAY cResponse LABEL "WS Response" WITH FRAME aaa.

DELETE OBJECT hPortType.
hWebSrvc:DISCONNECT().
DELETE OBJECT hWebSrvc.

/**************** Internal Procedures ****************/
PROCEDURE ReqHandler:
    /* Define procedure parameters */
    DEFINE OUTPUT PARAMETER hHeader    AS HANDLE.
    DEFINE INPUT  PARAMETER cNamespace AS CHARACTER.
    DEFINE INPUT  PARAMETER cLocalNS   AS CHARACTER.
    DEFINE OUTPUT PARAMETER lDeleteOnDone AS LOGICAL.

    /* Pass in global header reused for every request */
    hHeader = g_header.
    lDeleteOnDone = FALSE.
END PROCEDURE.

PROCEDURE BuildRequestHeader:
    /* Define procedure parameter */
    DEFINE OUTPUT PARAMETER hHeader AS HANDLE.

    DEFINE VARIABLE hHeaderEntryref AS HANDLE.
    DEFINE VARIABLE ClientNS AS CHARACTER initial "http://ServiceHost/SOAPHeader".

    /* Create SOAP header and server objects */
    CREATE SOAP-HEADER hHeader.
    CREATE SOAP-HEADER-ENTRYREF hHeaderEntryref.

    /* Create x-doc objects to build header */
    CREATE X-DOCUMENT hXdoc.
    CREATE X-NODEREF hXnoderef1.
    CREATE X-NODEREF hXnoderef2.
    CREATE X-NODEREF hXtext.

    /* Create the header entry */
    hHeader:ADD-HEADER-ENTRY(hHeaderEntryref).

    /* Create the header namespace data */
    hXdoc:CREATE-NODE-NAMESPACE(hXnoderef1, ClientNS, 
                                "AuthorizeHeader", "ELEMENT").
    hXdoc:CREATE-NODE-NAMESPACE(hXnoderef2, "http://www.w3.org/2000/xmlns/", 
                                "xmlns", "ATTRIBUTE").
    hXnoderef2:NODE-VALUE = ClientNS.
    hXnoderef1:SET-ATTRIBUTE-NODE(hXnoderef2).
    hXdoc:INSERT-BEFORE(hXnoderef1, ?).

    /* Create the Username/Password data */
    hXdoc:CREATE-NODE-NAMESPACE(hXnoderef2, ClientNS, 
                                "UserName", "ELEMENT").
    hXnoderef1:APPEND-CHILD(hXnoderef2).
    hXdoc:CREATE-NODE(hXtext,"","text").
    hXnoderef2:APPEND-CHILD(hXtext).
    hXtext:NODE-VALUE = cUsername.

    hXdoc:CREATE-NODE-NAMESPACE(hXnoderef2, ClientNS, 
                                "Password", "ELEMENT").
    hXnoderef1:APPEND-CHILD(hXnoderef2).
    hXdoc:CREATE-NODE(hXtext, "", "text").
    hXnoderef2:APPEND-CHILD(hXtext).
    hXtext:NODE-VALUE = cPassword.

    /* Fill the header entry using a deep copy */
    hHeaderEntryref:SET-NODE(hXnoderef1).

    /* Procedure/header cleanup */ 
    DELETE OBJECT hXdoc.
    DELETE OBJECT hXnoderef1.
    DELETE OBJECT hXnoderef2.
    DELETE OBJECT hXtext.
    DELETE OBJECT hHeaderEntryref.
END PROCEDURE.
